searchData={"items":[{"type":"module","title":"Flop.Phoenix","doc":"Phoenix components for pagination, sortable tables and filter forms with\n[Flop](https://hex.pm/packages/flop).","ref":"Flop.Phoenix.html"},{"type":"module","title":"Introduction - Flop.Phoenix","doc":"Please refer to the [Readme](README.md) for an introduction.","ref":"Flop.Phoenix.html#module-introduction"},{"type":"module","title":"Customization - Flop.Phoenix","doc":"To customize the components, it is recommended to define wrapper components in\nyour `CoreComponents` module that pass attributes that are constant for your\napplication and add additional markup as necessary.\n\nFor example, to customize the `pagination` component, define your own\n`pagination` component:\n\n```elixir\ndefmodule MyAppWeb.CoreComponents do\n  use Phoenix.Component\n\n  attr :meta, Flop.Meta, required: true\n  attr :path, :any, default: nil\n  attr :on_paginate, JS, default: nil\n  attr :target, :string, default: nil\n\n  attr :aria_label, :string,\n    default: \"Pagination\",\n    doc: \"\"\"\n    Aria label for the ` ` element. The value should be localized. In\n    languages with latin characters, the first letter should be capitalized.\n    If multiple pagination components are rendered on the same page, each one\n    should have a distinct aria label.\n    \"\"\"\n\n  def pagination(assigns) do\n    ~H\"\"\"\n     \n      <:previous attrs={[class: \"previous\"]}>\n         \n      </:previous>\n      <:next attrs={[class: \"next\"]}>\n         \n      </:next>\n      <:ellipsis>\n         ‥ \n      </:ellipsis>\n     \n    \"\"\"\n  end\nend\n```\n\nRefer to the documentation of `Flop.Phoenix.pagination/1` for available\nattributes and slots on the pagination component and to `t:table_option/0` for\na list of available options and defaults for the table component.","ref":"Flop.Phoenix.html#module-customization"},{"type":"module","title":"Using links - Flop.Phoenix","doc":"If the `path` attribute is set on the pagination and table component,\npagination and sorting is handled via query parameters. You will need to\nhandle those parameters in the `c:Phoenix.LiveView.handle_params/3` callback\nof your LiveView module.\n\n    def handle_params(params, _, socket) do\n      {pets, meta} = Pets.list_pets(params)\n      {:noreply, assign(socket, pets: pets, meta: meta)}\n    end","ref":"Flop.Phoenix.html#module-using-links"},{"type":"module","title":"Using JS commands - Flop.Phoenix","doc":"You can pass a `Phoenix.LiveView.JS` command as `on_paginate` and `on_sort`\nattributes.\n\nIf used with the `path` attribute, the URL will be patched _and_ the given\nJS command will be executed. This can be used to scroll to the top after a\npagination or sorting event, for example.\n\nIf used without the `path` attribute, you will need to include a `push`\ncommand to trigger an event when a pagination or sort link is clicked.\n\nYou can set a different target with the `target` attribute, which will be used\nas `phx-target`.\n\n```heex\n \n\n \n```\n\nYou will need to handle the event in the `c:Phoenix.LiveView.handle_event/3`\nor `c:Phoenix.LiveComponent.handle_event/3` callback of your\nLiveView or LiveComponent module.\n\n    # for page-based pagination\n    def handle_event(\"paginate-pets\", %{\"page\" => page}, socket) do\n      flop = Flop.set_page(socket.assigns.meta.flop, page)\n      {pets, meta} = Pets.list_pets(flop)\n      {:noreply, assign(socket, pets: pets, meta: meta)}\n    end\n\n    # for cursor-based pagination\n    def handle_event(\"paginate-pets\", %{\"to\" => direction}, socket) do\n      flop =\n        case direction do\n          :previous -> Flop.to_previous_cursor(socket.assigns.meta)\n          :next -> Flop.to_next_cursor(socket.assigns.meta)\n        end\n\n      {pets, meta} = Pets.list_pets(flop)\n      {:noreply, assign(socket, pets: pets, meta: meta)}\n    end\n\n    def handle_event(\"sort-pets\", %{\"order\" => order}, socket) do\n      flop = Flop.push_order(socket.assigns.meta.flop, order)\n      {pets, meta} = Pets.list_pets(flop)\n      {:noreply, assign(socket, pets: pets, meta: meta)}\n    end","ref":"Flop.Phoenix.html#module-using-js-commands"},{"type":"function","title":"Flop.Phoenix.build_path/3","doc":"Builds a path that includes query parameters for the given `Flop` struct\nusing the referenced Phoenix path helper function.\n\nThe first argument can be either one of:\n\n- an MFA tuple (module, function name as atom, arguments)\n- a 2-tuple (function, arguments)\n- a URL string, usually produced with a verified route (e.g. `~p\"/some/path\"`)\n- a function that takes the Flop parameters as a keyword list as an argument\n\nDefault values for `limit`, `page_size`, `order_by` and `order_directions` are\nomitted from the query parameters. To pick up the default parameters from a\nschema module deriving `Flop.Schema`, you need to pass the `:for` option. To\npick up the default parameters from the backend module, you need to pass the\n`:backend` option. If you pass a `Flop.Meta` struct as the second argument,\nthese options are retrieved from the struct automatically.\n\n> #### Date and Time Filters {: .info}\n>\n> When using filters on `Date`, `DateTime`, `NaiveDateTime` or `Time` fields,\n> you may need to implement the `Phoenix.Param` protocol for these structs.\n> See the documentation for `to_query/2`.","ref":"Flop.Phoenix.html#build_path/3"},{"type":"function","title":"Examples - Flop.Phoenix.build_path/3","doc":"","ref":"Flop.Phoenix.html#build_path/3-examples"},{"type":"function","title":"With a verified route - Flop.Phoenix.build_path/3","doc":"The examples below use plain URL strings without the p-sigil, so that the\ndoc tests work, but in your application, you can use verified routes or\nanything else that produces a URL.\n\n    iex> flop = %Flop{page: 2, page_size: 10}\n    iex> path = build_path(\"/pets\", flop)\n    iex> %URI{path: parsed_path, query: parsed_query} = URI.parse(path)\n    iex> {parsed_path, URI.decode_query(parsed_query)}\n    {\"/pets\", %{\"page\" => \"2\", \"page_size\" => \"10\"}}\n\nThe Flop query parameters will be merged into existing query parameters.\n\n    iex> flop = %Flop{page: 2, page_size: 10}\n    iex> path = build_path(\"/pets?species=dogs\", flop)\n    iex> %URI{path: parsed_path, query: parsed_query} = URI.parse(path)\n    iex> {parsed_path, URI.decode_query(parsed_query)}\n    {\"/pets\", %{\"page\" => \"2\", \"page_size\" => \"10\", \"species\" => \"dogs\"}}","ref":"Flop.Phoenix.html#build_path/3-with-a-verified-route"},{"type":"function","title":"With an MFA tuple - Flop.Phoenix.build_path/3","doc":"iex> flop = %Flop{page: 2, page_size: 10}\n    iex> build_path(\n    ...>   {Flop.PhoenixTest, :route_helper, [%Plug.Conn{}, :pets]},\n    ...>   flop\n    ...> )\n    \"/pets?page_size=10&page=2\"","ref":"Flop.Phoenix.html#build_path/3-with-an-mfa-tuple"},{"type":"function","title":"With a function/arguments tuple - Flop.Phoenix.build_path/3","doc":"iex> pet_path = fn _conn, :index, query ->\n    ...>   \"/pets?\" <> Plug.Conn.Query.encode(query)\n    ...> end\n    iex> flop = %Flop{page: 2, page_size: 10}\n    iex> build_path({pet_path, [%Plug.Conn{}, :index]}, flop)\n    \"/pets?page_size=10&page=2\"\n\nWe're defining fake path helpers for the scope of the doctests. In a real\nPhoenix application, you would pass something like\n`{Routes, :pet_path, args}` or `{&Routes.pet_path/3, args}` as the\nfirst argument.","ref":"Flop.Phoenix.html#build_path/3-with-a-function-arguments-tuple"},{"type":"function","title":"Passing a `Flop.Meta` struct or a keyword list - Flop.Phoenix.build_path/3","doc":"You can also pass a `Flop.Meta` struct or a keyword list as the third\nargument.\n\n    iex> pet_path = fn _conn, :index, query ->\n    ...>   \"/pets?\" <> Plug.Conn.Query.encode(query)\n    ...> end\n    iex> flop = %Flop{page: 2, page_size: 10}\n    iex> meta = %Flop.Meta{flop: flop}\n    iex> build_path({pet_path, [%Plug.Conn{}, :index]}, meta)\n    \"/pets?page_size=10&page=2\"\n    iex> query_params = to_query(flop)\n    iex> build_path({pet_path, [%Plug.Conn{}, :index]}, query_params)\n    \"/pets?page_size=10&page=2\"","ref":"Flop.Phoenix.html#build_path/3-passing-a-flop-meta-struct-or-a-keyword-list"},{"type":"function","title":"Additional path parameters - Flop.Phoenix.build_path/3","doc":"If the path helper takes additional path parameters, just add them to the\nsecond argument.\n\n    iex> user_pet_path = fn _conn, :index, id, query ->\n    ...>   \"/users/#{id}/pets?\" <> Plug.Conn.Query.encode(query)\n    ...> end\n    iex> flop = %Flop{page: 2, page_size: 10}\n    iex> build_path({user_pet_path, [%Plug.Conn{}, :index, 123]}, flop)\n    \"/users/123/pets?page_size=10&page=2\"","ref":"Flop.Phoenix.html#build_path/3-additional-path-parameters"},{"type":"function","title":"Additional query parameters - Flop.Phoenix.build_path/3","doc":"If the last path helper argument is a query parameter list, the Flop\nparameters are merged into it.\n\n    iex> pet_url = fn _conn, :index, query ->\n    ...>   \"https://pets.flop/pets?\" <> Plug.Conn.Query.encode(query)\n    ...> end\n    iex> flop = %Flop{order_by: :name, order_directions: [:desc]}\n    iex> build_path({pet_url, [%Plug.Conn{}, :index, [user_id: 123]]}, flop)\n    \"https://pets.flop/pets?user_id=123&order_directions[]=desc&order_by=name\"\n    iex> build_path(\n    ...>   {pet_url,\n    ...>    [%Plug.Conn{}, :index, [category: \"small\", user_id: 123]]},\n    ...>   flop\n    ...> )\n    \"https://pets.flop/pets?category=small&user_id=123&order_directions[]=desc&order_by=name\"","ref":"Flop.Phoenix.html#build_path/3-additional-query-parameters"},{"type":"function","title":"Set page as path parameter - Flop.Phoenix.build_path/3","doc":"Finally, you can also pass a function that takes the Flop parameters as\na keyword list as an argument. Default values will not be included in the\nparameters passed to the function. You can use this if you need to set some\nof the parameters as path parameters instead of query parameters.\n\n    iex> flop = %Flop{page: 2, page_size: 10}\n    iex> build_path(\n    ...>   fn params ->\n    ...>     {page, params} = Keyword.pop(params, :page)\n    ...>     query = Plug.Conn.Query.encode(params)\n    ...>     if page, do: \"/pets/page/#{page}?#{query}\", else: \"/pets?#{query}\"\n    ...>   end,\n    ...>   flop\n    ...> )\n    \"/pets/page/2?page_size=10\"\n\nNote that in this example, the anonymous function just returns a string. With\nPhoenix 1.7, you will be able to use verified routes.\n\n    build_path(\n      fn params ->\n        {page, query} = Keyword.pop(params, :page)\n        if page, do: ~p\"/pets/page/#{page}?#{query}\", else: ~p\"/pets?#{query}\"\n      end,\n      flop\n    )\n\nNote that the keyword list passed to the path builder function is built using\n`Plug.Conn.Query.encode/2`, which means filters are formatted as map with\ninteger keys.","ref":"Flop.Phoenix.html#build_path/3-set-page-as-path-parameter"},{"type":"function","title":"Set filter value as path parameter - Flop.Phoenix.build_path/3","doc":"If you need to set a filter value as a path parameter, you can use\n`Flop.Filter.pop/3`.\n\n    iex> flop = %Flop{\n    ...>   page: 5,\n    ...>   order_by: [:published_at],\n    ...>   filters: [\n    ...>     %Flop.Filter{field: :category, op: :==, value: \"announcements\"}\n    ...>   ]\n    ...> }\n    iex> build_path(\n    ...>   fn params ->\n    ...>     {page, params} = Keyword.pop(params, :page)\n    ...>     filters = Keyword.get(params, :filters, [])\n    ...>     {category, filters} = Flop.Filter.pop(filters, :category)\n    ...>     params = Keyword.put(params, :filters, filters)\n    ...>     query = Plug.Conn.Query.encode(params)\n    ...>\n    ...>     case {page, category} do\n    ...>       {nil, nil} -> \"/articles?#{query}\"\n    ...>       {page, nil} -> \"/articles/page/#{page}?#{query}\"\n    ...>       {nil, %{value: category}} -> \"/articles/category/#{category}?#{query}\"\n    ...>       {page, %{value: category}} -> \"/articles/category/#{category}/page/#{page}?#{query}\"\n    ...>     end\n    ...>   end,\n    ...>   flop\n    ...> )\n    \"/articles/category/announcements/page/5?order_by[]=published_at\"","ref":"Flop.Phoenix.html#build_path/3-set-filter-value-as-path-parameter"},{"type":"function","title":"Flop.Phoenix.filter_fields/1","doc":"Renders all inputs for a filter form including the hidden inputs.","ref":"Flop.Phoenix.html#filter_fields/1"},{"type":"function","title":"Example - Flop.Phoenix.filter_fields/1","doc":"def filter_form(%{meta: meta} = assigns) do\n      assigns = assign(assigns, :form, Phoenix.Component.to_form(meta))\n\n      ~H\"\"\"\n      <.form for={@form}>\n        <.filter_fields :let={i} form={@form} fields={[:email, :name]}>\n          <.input\n            field={i.field}\n            label={i.label}\n            type={i.type}\n            {i.rest}\n          />\n        </.filter_fields>\n      </.form>\n      \"\"\"\n    end\n\nThis assumes that you have defined an `input` component that renders a form\ninput including the label.\n\nThese options are passed to the inner block via `:let`:\n\n- The `field` is a `Phoenix.HTML.FormField` struct.\n- The `type` is the input type as a string (e.g. `\"text\"`, `\"number\"`). The\n  type is derived from the type of the field being filtered on, but it can\n  be overridden in the field options.\n- `rest` contains any additional field options passed.","ref":"Flop.Phoenix.html#filter_fields/1-example"},{"type":"function","title":"Field configuration - Flop.Phoenix.filter_fields/1","doc":"The fields can be passed as atoms or keywords with additional options.\n\n    fields={[:name, :email]}\n\nOr\n\n    fields={[\n      name: [label: gettext(\"Name\")],\n      email: [\n        label: gettext(\"Email\"),\n        op: :ilike_and,\n        type: \"email\"\n      ],\n      age: [\n        label: gettext(\"Age\"),\n        type: \"select\",\n        prompt: \"\",\n        options: [\n          {gettext(\"young\"), :young},\n          {gettext(\"old\"), :old)}\n        ]\n      ]\n    ]}\n\nAvailable options:\n\n- `label` - Defaults to the humanized field name.\n- `op` - Defaults to `:==`.\n- `type` - Defaults to an input type depending on the Ecto type of the filter\n  field.\n\nAny additional options will be passed to the input component (e.g. HTML\nclasses or a list of options).","ref":"Flop.Phoenix.html#filter_fields/1-field-configuration"},{"type":"function","title":"Attributes - Flop.Phoenix.filter_fields/1","doc":"* `form` (`Phoenix.HTML.Form`) (required)\n* `fields` (`:list`) - The list of fields and field options. Note that inputs will not be rendered\n  for fields that are not marked as filterable in the schema\n  (see `Flop.Schema`).\n\n  If `dynamic` is set to `false`, only fields in this list are rendered. If\n  `dynamic` is set to `true`, only fields for filters present in the given\n  `Flop.Meta` struct are rendered, and the fields are rendered even if they\n  are not passed in the `fields` list. In the latter case, `fields` is\n  optional, but you can still pass label and input configuration this way.\n\n  Note that in a dynamic form, it is not possible to configure a single field\n  multiple times.\n\n  Defaults to `[]`.\n* `dynamic` (`:boolean`) - If `true`, fields are only rendered for filters that are present in the\n  `Flop.Meta` struct passed to the form. You can use this for rendering filter\n  forms that allow the user to add and remove filters dynamically. The\n  `fields` assign is only used for looking up the options in that case.\n\n  Defaults to `false`.","ref":"Flop.Phoenix.html#filter_fields/1-attributes"},{"type":"function","title":"Slots - Flop.Phoenix.filter_fields/1","doc":"* `inner_block` - The necessary options for rendering a label and an input are passed to the\n  inner block, which allows you to render the fields with your existing\n  components.\n\n  ```heex\n  <.filter_fields :let={i} form={@form} fields={[:email, :name]}>\n    <.input\n      field={i.field}\n      label={i.label}\n      type={i.type}\n      {i.rest}\n    />\n  </.filter_fields>\n  ```\n\n  The options passed to the inner block are:\n\n  - `field` - A `Phoenix.HTML.FormField` struct.\n  - `type` - The input type as a string.\n  - `label` - The label text as a string.\n  - `rest` - Any additional options passed in the field options.","ref":"Flop.Phoenix.html#filter_fields/1-slots"},{"type":"function","title":"Flop.Phoenix.hidden_inputs_for_filter/1","doc":"Renders hidden inputs for the given form.\n\nYou can use this for convenience if you have a complex form layout that cannot\nbe accomplished with `Flop.Phoenix.filter_fields/1`. Put it as a direct child\nof the `form` component to render the hidden inputs for pagination and order\nparameters. Then use `Phoenix.Component.inputs_for/1` to render a single\nfilter field, and place this component within the anonymous function to render\nthe hidden inputs for the filter field and operator.\n\nSince the filters are represented as an array in the params, make sure to\nadd the `offset` option so that the `Flop.Meta` can be properly mapped back to\nyour input fields. For every call to `inputs_for` always add the length of all\nprevious calls to `inputs_for` as offset. Note that the example below uses\nthe old `Phoenix.Component.inputs_for/1` function.\nAlso don't forget to set the `skip_persistent_id` attribute\nto prevent LiveView from overriding the IDs and causing duplicate DOM IDs.\n\n```heex\n<.form :let={f} for={@meta}>\n  <.hidden_inputs_for_filter form={@form} />\n\n   \n     \n      <.inputs_for\n        :let={ff}\n        field={f[:filters]}\n        options={[fields: [:name]]}\n        skip_persistent_id\n      >\n        <.hidden_inputs_for_filter form={ff} />\n        <.input label=\"Name\" type=\"text\" field={ff[:value]} />\n      </.inputs_for>\n     \n     \n      <.inputs_for\n        :let={ff}\n        field={f[:filters]}\n        options={[fields: [email: [op: :ilike]], offset: 1]}\n        skip_persistent_id\n      >\n        <.hidden_inputs_for_filter form={ff} />\n        <.input label=\"E-mail\" type=\"email\" field={ff[:value]} />\n      </.inputs_for>\n     \n   \n</.form>\n```","ref":"Flop.Phoenix.html#hidden_inputs_for_filter/1"},{"type":"function","title":"Attributes - Flop.Phoenix.hidden_inputs_for_filter/1","doc":"* `form` (`Phoenix.HTML.Form`) (required)","ref":"Flop.Phoenix.html#hidden_inputs_for_filter/1-attributes"},{"type":"function","title":"Flop.Phoenix.page_link_aria_label/1","doc":"Returns an aria label for a link to the given page number.\n\nThis is the default function used by `pagination/1`.","ref":"Flop.Phoenix.html#page_link_aria_label/1"},{"type":"function","title":"Example - Flop.Phoenix.page_link_aria_label/1","doc":"iex> page_link_aria_label(5)\n    \"Go to page 5\"","ref":"Flop.Phoenix.html#page_link_aria_label/1-example"},{"type":"type","title":"Flop.Phoenix.page_link_option/0","doc":"Defines how many page links to render.\n\n- `:all` - Renders all page links.\n- `:none` - Does not render any page links.\n- Integer - Renders up to the specified number of page links in addition to\n  the first and last page.","ref":"Flop.Phoenix.html#t:page_link_option/0"},{"type":"function","title":"Flop.Phoenix.page_link_range/3","doc":"Returns the range of page links to be rendered.","ref":"Flop.Phoenix.html#page_link_range/3"},{"type":"function","title":"Usage - Flop.Phoenix.page_link_range/3","doc":"iex> page_link_range(:all, 4, 20)\n    {1, 20}\n\n    iex> page_link_range(:none, 4, 20)\n    {nil, nil}\n\n    iex> page_link_range(5, 4, 20)\n    {2, 6}","ref":"Flop.Phoenix.html#page_link_range/3-usage"},{"type":"function","title":"Flop.Phoenix.pagination/1","doc":"Renders pagination links for the given `Flop.Meta` struct.\n\nThis component can render both page-based or cursor-based pagination links.\nWhich one is used depends on the pagination type that was used to make the\nFlop query.","ref":"Flop.Phoenix.html#pagination/1"},{"type":"function","title":"Examples - Flop.Phoenix.pagination/1","doc":"With a verified route:\n\n```heex\n \n```\n\nWith an event:\n\n```heex\n \n```\n\nWith a route helper:\n\n```heex\n \n```\n\nWith all attributes and slots:\n\n```heex\n \n  <:previous attrs={[class: \"pagination-previous\"]}>\n    Previous\n  </:previous>\n  <:next attrs={[class: \"pagination-next\"]}>\n    Next\n  </:next>\n  <:ellipsis>\n     &hellip; \n  </:ellipsis>\n \n```","ref":"Flop.Phoenix.html#pagination/1-examples"},{"type":"function","title":"Attributes - Flop.Phoenix.pagination/1","doc":"* `meta` (`Flop.Meta`) (required) - The meta information of the query as returned by the `Flop` query functions.\n* `path` (`:any`) - If set, the current view is patched with updated query parameters when a\n  pagination link is clicked. In case the `on_paginate` attribute is set as\n  well, the URL is patched _and_ the given command is executed.\n\n  The value must be either a URI string (Phoenix verified route), an MFA or FA\n  tuple (Phoenix route helper), or a 1-ary path builder function. See\n  `Flop.Phoenix.build_path/3` for details.\n\n  Defaults to `nil`.\n* `on_paginate` (`Phoenix.LiveView.JS`) - A `Phoenix.LiveView.JS` command that is triggered when a pagination link is\n  clicked.\n\n  If used without the `path` attribute, you should include a `push` operation\n  to handle the event with the `handle_event` callback.\n\n  ```heex\n  <.pagination\n    meta={@meta}\n    on_paginate={\n      JS.dispatch(\"my_app:scroll_to\", to: \"#pet-table\") |> JS.push(\"paginate\")\n    }\n  />\n  ```\n\n  If used with the `path` attribute, the URL is patched _and_ the given\n  JS command is executed.\n\n  ```heex\n  <.pagination\n    meta={@meta}\n    path={~p\"/pets\"}\n    on_paginate={JS.dispatch(\"my_app:scroll_to\", to: \"#pet-table\")}\n  />\n  ```\n\n  With the above attributes in place, you can add the following JavaScript to\n  your application to scroll to the top of your table whenever a pagination\n  link is clicked:\n\n  ```js\n  window.addEventListener(\"my_app:scroll_to\", (e) => {\n    e.target.scrollIntoView();\n  });\n  ```\n\n  You can use CSS to scroll to the new position smoothly.\n\n  ```css\n  html {\n    scroll-behavior: smooth;\n  }\n  ```\n\n  Defaults to `nil`.\n* `target` (`:string`) - Sets the `phx-target` attribute for the pagination links. Defaults to `nil`.\n* `page_link_aria_label_fun` (`{:fun, 1}`) - Function that returns an aria label for the page link or button to the\n  given page number.\n\n  The returned label should be localized and start with a capital letter.\n\n  Defaults to `&Flop.Phoenix.page_link_aria_label/1`.\n* `page_links` (`:any`) - Defines how many page links to render.\n\n  - `:all` - Renders all page links.\n  - `:none` - Does not render any page links.\n  - Integer - Renders up to the specified number of page links in addition to\n    the first and last page.\n\n  This attribute is only used for page-based pagination.\n\n  Defaults to `5`.\n* `reverse` (`:boolean`) - By default, the `next` link moves forward with the `:after` parameter set to\n  the end cursor, and the `previous` link moves backward with the `:before`\n  parameter set to the start cursor. If `reverse` is set to `true`, the\n  destinations of the links are switched.\n\n  This attribute is only for cursor-based pagination.\n\n  Defaults to `false`.\n* `page_list_attrs` (`:list`) - Attributes to be added to the ` ` that contains the page links. Defaults to `[]`.\n* `page_list_item_attrs` (`:list`) - Attributes to be added to the ` ` elements that contain the page links. Defaults to `[]`.\n* `page_link_attrs` (`:list`) - Attributes to be added to the page links or buttons.\n\n  These attributes are not applied to previous links, next links, or the\n  current page link.\n\n  Defaults to `[]`.\n* `current_page_link_attrs` (`:list`) - Attributes to be added to the current page link or button.\n\n  Note that the `aria-current` attribute is automatically set.\n\n  It is recommended to define CSS styles using the `[aria-current=\"page\"]`\n  selector instead of using a class.\n\n  Defaults to `[]`.\n* `disabled_link_attrs` (`:list`) - Attributes to be added to disabled previous/next links or buttons.\n\n  If a `class` is set, it is merged with the class set on the previous/next\n  slot.\n\n  Note that the `disabled` attribute is automatically set for buttons and the\n  `aria-disabled=\"true\"` attribute is automatically set for links.\n\n  It is recommended to define CSS styles using the\n  `[disabled], [aria-disabled=\"true\"]` selector instead of using a class.\n\n  Defaults to `[]`.\n* Global attributes are accepted. The attributes are added to the outer ` ` element.\n\n\n  The `aria-label` defaults to `\"Pagination\"`. If your application is\n  localized, the label should be translated to the user locale. In languages\n  with latin characters, the first letter should be capitalized. If multiple\n  pagination components are rendered on the same page, each one should have\n  a distinct aria label.","ref":"Flop.Phoenix.html#pagination/1-attributes"},{"type":"function","title":"Slots - Flop.Phoenix.pagination/1","doc":"* `previous` - The content of the pagination link or button to the previous page.\n\n  If the slot is not used, the text \"Previous\" is rendered.\n\n  Accepts attributes:\n\n  * `attrs` (`:list`) - Any additional attributes to add to the link or button.\n\n    Defaults to `[aria: [label: \"Go to previous page\"]]`.\n\n* `next` - The content of the pagination link or button to the next page.\n\n  If the slot is not used, the text \"Next\" is rendered.\n\n  Accepts attributes:\n\n  * `attrs` (`:list`) - Any additional attributes to add to the link or button.\n\n    Defaults to `[aria: [label: \"Go to next page\"]]`.\n\n* `ellipsis` - The content of the ` ` element that usually shows an ellipsis and is\n  rendered toward the beginning and/or end of the page links if there are\n  more pages than the configured limit.\n\n  If the slot is not used, a default element is used:\n\n  ```html\n   &hellip; \n  ```","ref":"Flop.Phoenix.html#pagination/1-slots"},{"type":"function","title":"Flop.Phoenix.pagination_for/1","doc":"This component is a pagination builder.\n\nIt does not render anything by itself. Instead, it prepares all the necessary\ninformation needed to render a pagination component and passes it to the\ninner block.\n\nFor an example implementation, see `pagination/1`.","ref":"Flop.Phoenix.html#pagination_for/1"},{"type":"function","title":"Example - Flop.Phoenix.pagination_for/1","doc":"```heex\n<.pagination_for\n  :let={p}\n  meta={@meta}\n  page_links={4}\n  path={~p\"/birds\"}\n>\n  <%!-- put together your component here --%>\n</.pagination_for>\n```\n\nThe variable passed to the inner block is a `Flop.Phoenix.Pagination` struct.","ref":"Flop.Phoenix.html#pagination_for/1-example"},{"type":"function","title":"Attributes - Flop.Phoenix.pagination_for/1","doc":"* `meta` (`Flop.Meta`) (required) - The meta information of the query as returned by the `Flop` query functions.\n* `path` (`:any`) - If set, a function that takes a page number and returns a link with\n  pagination, filter, and sort parameters based on the given path is passed\n  as `path_fun` to the inner block.\n\n  The value must be either a URI string (Phoenix verified route), an MFA or FA\n  tuple (Phoenix route helper), or a 1-ary path builder function. See\n  `Flop.Phoenix.build_path/3` for details.\n\n  Defaults to `nil`.\n* `page_links` (`:any`) - Defines how many page links to render.\n\n  - `:all` - Renders all page links.\n  - `:none` - Does not render any page links.\n  - Integer - Renders up to the specified number of page links in addition to\n    the first and last page.\n\n  A `page_range_start` and `page_range_end` attribute are passed to the\n  inner block based on this option. If this attribute is set to `:none`, both\n  of those values will be `nil`.\n\n  Defaults to `5`.\n* `reverse` (`:boolean`) - By default, the `next` link moves forward with the `:after` parameter set to\n  the end cursor, and the `previous` link moves backward with the `:before`\n  parameter set to the start cursor. If `reverse` is set to `true`, the\n  destinations of the links are switched.\n\n  Defaults to `false`.","ref":"Flop.Phoenix.html#pagination_for/1-attributes"},{"type":"function","title":"Slots - Flop.Phoenix.pagination_for/1","doc":"* `inner_block` (required)","ref":"Flop.Phoenix.html#pagination_for/1-slots"},{"type":"function","title":"Flop.Phoenix.table/1","doc":"Generates a table with sortable columns.","ref":"Flop.Phoenix.html#table/1"},{"type":"function","title":"Example - Flop.Phoenix.table/1","doc":"```heex\n \n  <:col :let={pet} label=\"Name\" field={:name}>{pet.name}</:col>\n  <:col :let={pet} label=\"Age\" field={:age}>{pet.age}</:col>\n \n```","ref":"Flop.Phoenix.html#table/1-example"},{"type":"function","title":"Flop.Schema - Flop.Phoenix.table/1","doc":"If you pass the `for` option when making the query with Flop, Flop Phoenix can\ndetermine which table columns are sortable. It also hides the `order` and\n`page_size` parameters if they match the default values defined with\n`Flop.Schema`.","ref":"Flop.Phoenix.html#table/1-flop-schema"},{"type":"function","title":"Attributes - Flop.Phoenix.table/1","doc":"* `id` (`:string`) - ID used on the table. If not set, an ID is chosen based on the schema\n  module derived from the `Flop.Meta` struct.\n\n  The ID is necessary in case the table is fed with a LiveView stream.\n\n* `items` (`:list`) (required) - The list of items to be displayed in rows. This is the result list returned\n  by the query.\n\n* `meta` (`Flop.Meta`) (required) - The `Flop.Meta` struct returned by the query function.\n* `path` (`:any`) - If set, the current view is patched with updated query parameters when a\n  header link for sorting is clicked. In case the `on_sort` attribute is\n  set as well, the URL is patched _and_ the given JS command is executed.\n\n  The value must be either a URI string (Phoenix verified route), an MFA or FA\n  tuple (Phoenix route helper), or a 1-ary path builder function. See\n  `Flop.Phoenix.build_path/3` for details.\n\n  Defaults to `nil`.\n* `on_sort` (`Phoenix.LiveView.JS`) - A `Phoenix.LiveView.JS` command that is triggered when a header link for\n  sorting is clicked.\n\n  If used without the `path` attribute, you should include a `push` operation\n  to handle the event with the `handle_event` callback.\n\n  ```heex\n  <.table\n    items={@items}\n    meta={@meta}\n    on_sort={\n      JS.dispatch(\"my_app:scroll_to\", to: \"#pet-table\") |> JS.push(\"sort\")\n    }\n  />\n  ```\n\n  If used with the `path` attribute, the URL is patched _and_ the given\n  JS command is executed.\n\n  ```heex\n  <.table\n    meta={@meta}\n    path={~p\"/pets\"}\n    on_sort={JS.dispatch(\"my_app:scroll_to\", to: \"#pet-table\")}\n  />\n  ```\n\n  Defaults to `nil`.\n* `target` (`:string`) - Sets the `phx-target` attribute for the header links. Defaults to `nil`.\n* `caption` (`:string`) - Content for the ` ` element. Defaults to `nil`.\n* `opts` (`:list`) - Keyword list with additional options (see `t:Flop.Phoenix.table_option/0`).\n  Note that the options passed to the function are deep merged into the\n  default options. Since these options will likely be the same for all the\n  tables in a project, it is recommended to define them once in a function or\n  set them in a wrapper function as described in the `Customization` section\n  of the module documentation.\n\n  Defaults to `[]`.\n* `row_id` (`:any`) - Overrides the default function that retrieves the row ID from a stream item. Defaults to `nil`.\n* `row_click` (`:any`) - Sets the `phx-click` function attribute for each row `td`. Expects to be a\n  function that receives a row item as an argument. This does not add the\n  `phx-click` attribute to the `action` slot.\n\n  Example:\n\n  ```heex\n  row_click={&JS.navigate(~p\"/users/#{&1}\")}\n  ```\n\n  Defaults to `nil`.\n* `row_item` (`:any`) - This function is called on the row item before it is passed to the :col\n  and :action slots.\n\n  Defaults to `&Function.identity/1`.","ref":"Flop.Phoenix.html#table/1-attributes"},{"type":"function","title":"Slots - Flop.Phoenix.table/1","doc":"* `col` (required) - For each column to render, add one `<:col>` element.\n\n  ```heex\n  <:col :let={pet} label=\"Name\" field={:name} col_style=\"width: 20%;\">\n    {pet.name}\n  </:col>\n  ```\n\n  Any additional assigns will be added as attributes to the ` ` elements.\n\n  Accepts attributes:\n\n  * `label` (`:any`) - The content for the header column.\n  * `field` (`:atom`) - The field name for sorting. If set and the field is configured as sortable\n    in the schema, the column header will be clickable, allowing the user to\n    sort by that column. If the field is not marked as sortable or if the\n    `field` attribute is omitted or set to `nil` or `false`, the column header\n    will not be clickable.\n\n  * `directions` (`:any`) - An optional 2-element tuple used for custom ascending and descending sort\n    behavior for the column, i.e. `{:asc_nulls_last, :desc_nulls_first}`\n\n  * `col_style` (`:string`) - If set, a ` ` element is rendered and the value of the\n    `col_style` assign is set as `style` attribute for the ` ` element of\n    the respective column. You can set the `width`, `background`, `border`,\n    and `visibility` of a column this way.\n\n  * `col_class` (`:string`) - If set, a ` ` element is rendered and the value of the\n    `col_class` assign is set as `class` attribute for the ` ` element of\n    the respective column. You can set the `width`, `background`, `border`,\n    and `visibility` of a column this way.\n\n  * `thead_th_attrs` (`:list`) - Additional attributes to pass to the ` ` element as a static keyword\n    list. Note that these attributes will override any conflicting\n    `thead_th_attrs` that are set at the table level.\n\n  * `th_wrapper_attrs` (`:list`) - Additional attributes for the ` ` element that wraps the\n    header link and the order direction symbol. Note that these attributes\n    will override any conflicting `th_wrapper_attrs` that are set at the table\n    level.\n\n  * `tbody_td_attrs` (`:any`) - Additional attributes to pass to the ` ` element. May be provided as a\n    static keyword list, or as a 1-arity function to dynamically generate the\n    list using row data. Note that these attributes will override any\n    conflicting `tbody_td_attrs` that are set at the table level.\n\n* `action` - The slot for showing user actions in the last table column. These columns\n  do not receive the `row_click` attribute.\n\n\n  ```heex\n  <:action :let={user}>\n    <.link navigate={~p\"/users/#{user}\"}>Show</.link>\n  </:action>\n  ```\n\n  Accepts attributes:\n\n  * `label` (`:string`) - The content for the header column.\n  * `col_style` (`:string`) - If set, a ` ` element is rendered and the value of the\n    `col_style` assign is set as `style` attribute for the ` ` element of\n    the respective column. You can set the `width`, `background`, `border`,\n    and `visibility` of a column this way.\n\n  * `col_class` (`:string`) - If set, a ` ` element is rendered and the value of the\n    `col_class` assign is set as `class` attribute for the ` ` element of\n    the respective column. You can set the `width`, `background`, `border`,\n    and `visibility` of a column this way.\n\n  * `thead_th_attrs` (`:list`) - Any additional attributes to pass to the ` ` as a keyword list.\n  * `tbody_td_attrs` (`:any`) - Any additional attributes to pass to the ` `. Can be a keyword list or\n    a function that takes the current row item as an argument and returns a\n    keyword list.\n\n* `foot` - You can optionally add a `foot`. The inner block will be rendered inside\n  a `tfoot` element.\n\n  ```heex\n   \n    <:foot>\n        Total:  {@total}   \n    </:foot>\n   \n  ```","ref":"Flop.Phoenix.html#table/1-slots"},{"type":"type","title":"Flop.Phoenix.table_option/0","doc":"Defines the available options for `Flop.Phoenix.table/1`.\n\n- `:container` - Wraps the table in a ` ` if `true`.\n  Default: `false`.\n- `:container_attrs` - The attributes for the table container.\n  Default: `[class: \"table-container\"]`.\n- `:no_results_content` - Any content that should be rendered if there are no\n  results. Default: ` No results. `.\n- `:table_attrs` - The attributes for the ` ` element.\n  Default: `[]`.\n- `:th_wrapper_attrs` - The attributes for the ` ` element that wraps the\n  header link and the order direction symbol.\n  Default: `[]`.\n- `:symbol_asc` - The symbol that is used to indicate that the column is\n  sorted in ascending order.\n  Default: `\"▴\"`.\n- `:symbol_attrs` - The attributes for the ` ` element that wraps the\n  order direction indicator in the header columns.\n  Default: `[class: \"order-direction\"]`.\n- `:symbol_desc` - The symbol that is used to indicate that the column is\n  sorted in ascending order.\n  Default: `\"▾\"`.\n- `:symbol_unsorted` - The symbol that is used to indicate that the column is\n  not sorted. Default: `nil`.\n- `:tbody_attrs`: Attributes to be added to the ` ` tag within the\n  ` `. Default: `[]`.\n- `:tbody_td_attrs`: Attributes to be added to each ` ` tag within the\n  ` `. Default: `[]`.\n- `:thead_attrs`: Attributes to be added to the ` ` tag within the\n  ` `. Default: `[]`.\n- `:tbody_tr_attrs`: Attributes to be added to each ` ` tag within the\n  ` `. A function with arity of 1 may be passed to dynamically generate\n  the attrs based on row data.\n  Default: `[]`.\n- `:thead_th_attrs`: Attributes to be added to each ` ` tag within the\n  ` `. Default: `[]`.\n- `:thead_tr_attrs`: Attributes to be added to each ` ` tag within the\n  ` `. Default: `[]`.","ref":"Flop.Phoenix.html#t:table_option/0"},{"type":"function","title":"Flop.Phoenix.to_query/2","doc":"Converts a Flop struct into a keyword list that can be used as a query with\nPhoenix verified routes or route helper functions.","ref":"Flop.Phoenix.html#to_query/2"},{"type":"function","title":"Default parameters - Flop.Phoenix.to_query/2","doc":"Default parameters for the limit and order parameters are omitted. The\ndefaults are determined by calling `Flop.get_option/3`.\n\n- Pass the `:for` option to pick up the default values from a schema module\n  deriving `Flop.Schema`.\n- Pass the `:backend` option to pick up the default values from your backend\n  configuration.\n- If neither the schema module nor the backend module have default options\n  set, the function will fall back to the application environment.","ref":"Flop.Phoenix.html#to_query/2-default-parameters"},{"type":"function","title":"Encoding queries - Flop.Phoenix.to_query/2","doc":"To encode the returned query as a string, you will need to use\n`Plug.Conn.Query.encode/1`. `URI.encode_query/1` does not support bracket\nnotation for arrays and maps.","ref":"Flop.Phoenix.html#to_query/2-encoding-queries"},{"type":"function","title":"Date and time filters - Flop.Phoenix.to_query/2","doc":"If you use the result of this function directly with\n`Phoenix.VerifiedRoutes.sigil_p/2` for verified routes or in a route helper\nfunction, all cast filter values need to be able to be converted to a string\nusing the `Phoenix.Param` protocol.\n\nThis protocol is implemented by default for integers, binaries, atoms, and\nstructs. For structs, Phoenix's default behavior is to fetch the id field.\n\nIf you have filters with `Date`, `DateTime`, `NaiveDateTime`,\n`Time` values, or any other custom structs (e.g. structs that represent\ncomposite types like a range column), you will need to implement the protocol\nfor these specific structs in your application.\n\n    defimpl Phoenix.Param, for: Date do\n      def to_param(%Date{} = d), do: to_string(d)\n    end\n\n    defimpl Phoenix.Param, for: DateTime do\n      def to_param(%DateTime{} = dt), do: to_string(dt)\n    end\n\n    defimpl Phoenix.Param, for: NaiveDateTime do\n      def to_param(%NaiveDateTime{} = dt), do: to_string(dt)\n    end\n\n    defimpl Phoenix.Param, for: Time do\n      def to_param(%Time{} = t), do: to_string(t)\n    end\n\nIt is important that the chosen string representation can be cast back into\nthe Ecto type.","ref":"Flop.Phoenix.html#to_query/2-date-and-time-filters"},{"type":"function","title":"Examples - Flop.Phoenix.to_query/2","doc":"iex> to_query(%Flop{})\n    []\n\n    iex> f = %Flop{page: 5, page_size: 20}\n    iex> to_query(f)\n    [page_size: 20, page: 5]\n\n    iex> f = %Flop{first: 20, after: \"g3QAAAABZAAEbmFtZW0AAAAFQXBwbGU=\"}\n    iex> to_query(f)\n    [first: 20, after: \"g3QAAAABZAAEbmFtZW0AAAAFQXBwbGU=\"]\n\n    iex> f = %Flop{\n    ...>   filters: [\n    ...>     %Flop.Filter{field: :name, op: :=~, value: \"Mag\"},\n    ...>     %Flop.Filter{field: :age, op: :>, value: 25}\n    ...>   ]\n    ...> }\n    iex> to_query(f)\n    [\n      filters: %{\n        0 => %{field: :name, op: :=~, value: \"Mag\"},\n        1 => %{field: :age, op: :>, value: 25}\n      }\n    ]\n    iex> to_query(f)\n    [filters: %{0 => %{value: \"Mag\", op: :=~, field: :name}, 1 => %{value: 25, op: :>, field: :age}}]\n\n    iex> f = %Flop{page: 5, page_size: 20}\n    iex> to_query(f, default_limit: 20)\n    [page: 5]\n\nEncoding the query as a string:\n\n    iex> f = %Flop{order_by: [:name, :age], order_directions: [:desc, :asc]}\n    iex> to_query(f)\n    [order_directions: [:desc, :asc], order_by: [:name, :age]]\n    iex> f |> to_query |> Plug.Conn.Query.encode()\n    \"order_directions[]=desc&order_directions[]=asc&order_by[]=name&order_by[]=age\"","ref":"Flop.Phoenix.html#to_query/2-examples"},{"type":"module","title":"Flop.Phoenix.Pagination","doc":"Defines a struct that holds the information needed to render a pagination\ncomponent.","ref":"Flop.Phoenix.Pagination.html"},{"type":"function","title":"Flop.Phoenix.Pagination.new/2","doc":"Returns a `Pagination` struct for the given `Flop.Meta` struct.","ref":"Flop.Phoenix.Pagination.html#new/2"},{"type":"function","title":"Options - Flop.Phoenix.Pagination.new/2","doc":"- `page_links` - Defines how many page links to render. Only used for\n  page-based pagination. Default: `5`.\n- `path` - The path to the current page in the format as accepted by\n  `Flop.Phoenix.build_path/3`. Default: `nil`.\n- `reverse` - Reverses the position of the previous and next link. Only used\n  for cursor-based pagination. Default: `false`.","ref":"Flop.Phoenix.Pagination.html#new/2-options"},{"type":"type","title":"Flop.Phoenix.Pagination.t/0","doc":"Describes the data needed to render a pagination component.","ref":"Flop.Phoenix.Pagination.html#t:t/0"},{"type":"type","title":"For page-based pagination - Flop.Phoenix.Pagination.t/0","doc":"- `current_page`\n- `ellipsis_end?` - Whether an ellipsis should be rendered between the middle\n  pagination links and the link to the last page.\n- `ellipsis_end?` - Whether an ellipsis should be rendered between the link to\n  the first page and the middle pagination links.\n- `next_page`\n- `page_range_start`, `page_range_end` - The range for the links for\n  individual pages.\n- `pagination_type` - In the case of page-based pagination, this is either\n  `:page` or `:offset`.\n- `path_fun` - 1-arity function that takes a page number and returns a path\n  to that page that also includes query parameters for filters and sorting.\n- `previous_page`\n- `total_pages`","ref":"Flop.Phoenix.Pagination.html#t:t/0-for-page-based-pagination"},{"type":"type","title":"For cursor-based pagination - Flop.Phoenix.Pagination.t/0","doc":"- `next_cursor` - The cursor to be used for the link to the next page.\n  Depending on the value of the `reverse` option, this is either the start\n  cursor or the end cursor of the `Flop.Meta` struct.\n- `next_direction` - The pagination direction for the link to the next page.\n  If the `reverse` option is set to `true`, this will be `:previous`.\n- `pagination_type` - In the case of cursor-based pagination, this is either\n  `:first` or `:last`.\n- `path_fun` - 2-arity function that takes a cursor and a direction and\n  returns a path to that page that also includes query parameters for filters\n  and sorting.\n- `previous_cursor` - The cursor to be used for the link to the previous page.\n  Depending on the value of the `reverse` option, this is either the start\n  cursor or the end cursor of the `Flop.Meta` struct.","ref":"Flop.Phoenix.Pagination.html#t:t/0-for-cursor-based-pagination"},{"type":"exception","title":"Flop.Phoenix.InvalidFilterFieldConfigError","doc":"Raised when the `fields` attribute of the `Flop.Phoenix.filter_fields`\ncomponent is set to an invalid value.","ref":"Flop.Phoenix.InvalidFilterFieldConfigError.html"},{"type":"function","title":"Flop.Phoenix.InvalidFilterFieldConfigError.message/1","doc":"","ref":"Flop.Phoenix.InvalidFilterFieldConfigError.html#message/1"},{"type":"exception","title":"Flop.Phoenix.NoMetaFormError","doc":"Raised when a `Phoenix.HTML.Form` struct is passed to\n`Flop.Phoenix.filter_fields/1` that was not built with `Flop.Meta` struct.","ref":"Flop.Phoenix.NoMetaFormError.html"},{"type":"function","title":"Flop.Phoenix.NoMetaFormError.message/1","doc":"","ref":"Flop.Phoenix.NoMetaFormError.html#message/1"},{"type":"exception","title":"Flop.Phoenix.PathOrJSError","doc":"Raised when a neither the `path` nor the `on_*` attribute is set for a\npagination or table component.","ref":"Flop.Phoenix.PathOrJSError.html"},{"type":"function","title":"Flop.Phoenix.PathOrJSError.message/1","doc":"","ref":"Flop.Phoenix.PathOrJSError.html#message/1"},{"type":"extras","title":"Flop Phoenix","doc":"# Flop Phoenix\n\n![CI](https://github.com/woylie/flop_phoenix/workflows/CI/badge.svg) [![Hex](https://img.shields.io/hexpm/v/flop_phoenix)](https://hex.pm/packages/flop_phoenix) [![Coverage Status](https://coveralls.io/repos/github/woylie/flop_phoenix/badge.svg)](https://coveralls.io/github/woylie/flop_phoenix)\n\nFlop Phoenix provides Phoenix components for pagination, sortable tables, and\nfilter forms with [Flop](https://hex.pm/packages/flop) and\n[Ecto](https://hex.pm/packages/ecto).","ref":"readme.html"},{"type":"extras","title":"Installation - Flop Phoenix","doc":"Add `flop_phoenix` to your list of dependencies in the `mix.exs` of your Phoenix\napplication.\n\n```elixir\ndef deps do\n  [\n    {:flop_phoenix, \"~> 0.25.0\"}\n  ]\nend\n```\n\nNext, set up your business logic following the\n[Flop documentation](https://hex.pm/packages/flop).","ref":"readme.html#installation"},{"type":"extras","title":"Context - Flop Phoenix","doc":"Define a context function that performs a list query using Flop.\n\n```elixir\ndefmodule MyApp.Pets do\n  alias MyApp.Pet\n\n  def list_pets(params) do\n    Flop.validate_and_run!(Pet, params, for: Pet, replace_invalid_params: true)\n  end\nend\n```\n\nNote the usage of the `replace_invalid_params` option, which lets Flop ignore\ninvalid parameters instead of producing an error.","ref":"readme.html#context"},{"type":"extras","title":"LiveView - Flop Phoenix","doc":"In the `handle_params` function of your LiveView module, pass the parameters\nto the list function to fetch the data and assign both the data and the meta\nstruct to the socket.\n\n```elixir\ndefmodule MyAppWeb.PetLive.Index do\n  use MyAppWeb, :live_view\n\n  alias MyApp.Pets\n\n  @impl Phoenix.LiveView\n  def handle_params(params, _, socket) do\n    {pets, meta} = Pets.list_pets(params)\n    {:noreply, assign(socket, pets: pets, meta: meta)}\n  end\nend\n```","ref":"readme.html#liveview"},{"type":"extras","title":"Sortable table and pagination - Flop Phoenix","doc":"Add a sortable table and pagination to your HEEx template:\n\n```heex\n \n  <:col :let={pet} label=\"Name\" field={:name}>{pet.name}</:col>\n  <:col :let={pet} label=\"Age\" field={:age}>{pet.age}</:col>\n \n\n \n```\n\nThe `path` attribute points to the current route, and Flop Phoenix appends\npagination, filtering, and sorting parameters to it. You can use verified\nroutes, route helpers, or custom path builder functions. For a\ndescription of the different formats, refer to the documentation of\n`Flop.Phoenix.build_path/3`.\n\nThe `field` attribute in the `:col` slot is optional. If set and the field\nis defined as sortable in the Ecto schema, the table header for\nthat column will be interactive, allowing users to sort by that column. However,\nif the field isn't defined as sortable, or if the field attribute is omitted, or\nset to `nil` or `false`, the table header will not be clickable.\n\nBy using the `for` option in your Flop query, Flop Phoenix can identify which\ntable columns are sortable. Additionally, it omits the `order` and `page_size`\nparameters if they match the default values defined with `Flop.Schema`.\n\nYou also have the option to pass a `Phoenix.LiveView.JS` command instead of or\nin addition to a path. For more details, please refer to the component\ndocumentation.\n\nThe pagination component can be used for both page-based pagination and\ncursor-based pagination. It chooses the pagination type based on the information\nfrom the `Flop.Meta` struct.","ref":"readme.html#sortable-table-and-pagination"},{"type":"extras","title":"Event-based pagination and sorting - Flop Phoenix","doc":"In the example above, the pagination, sorting, and filtering parameters are\nappended to the URL as query parameters. Most of the time, this provides a\nbetter user experience, since users are able to bookmark or share a URL that\nleads to the exact view.\n\nIn some cases, though, you may prefer not to handle the parameters via the URL.\nFor example, you may have multiple pageable areas in a single view, or a\npageable widget that is not part of the main content.\n\nIn that case, you can set the `on_paginate` and `on_sort` attributes instead of\nthe `path` attribute and handle these events with the `handle_event` callback.\n\nRefer to the \"Using JS commands\" section in the `Flop.Phoenix` module\ndocumentation for an example.","ref":"readme.html#event-based-pagination-and-sorting"},{"type":"extras","title":"Filter forms - Flop Phoenix","doc":"Flop Phoenix implements the `Phoenix.HTML.FormData` for the `Flop.Meta` struct.\nAs such, you can easily pass the struct to Phoenix form functions. One\nstraightforward way to render a filter form is through the\n`Flop.Phoenix.filter_fields/1` component, as shown below:\n\n```elixir\nattr :fields, :list, required: true\nattr :meta, Flop.Meta, required: true\nattr :id, :string, default: nil\nattr :on_change, :string, default: \"update-filter\"\nattr :target, :string, default: nil\n\ndef filter_form(%{meta: meta} = assigns) do\n  assigns = assign(assigns, form: Phoenix.Component.to_form(meta), meta: nil)\n\n  ~H\"\"\"\n  <.form\n    for={@form}\n    id={@id}\n    phx-target={@target}\n    phx-change={@on_change}\n    phx-submit={@on_change}\n  >\n    <.filter_fields :let={i} form={@form} fields={@fields}>\n      <.input\n        field={i.field}\n        label={i.label}\n        type={i.type}\n        phx-debounce={120}\n        {i.rest}\n      />\n    </.filter_fields>\n\n     reset \n  </.form>\n  \"\"\"\nend\n```\n\nNow you can render a filter form like this:\n\n```heex\n<.filter_form\n  fields={[:name, :email]}\n  meta={@meta}\n  id=\"user-filter-form\"\n/>\n```\n\nYou will need to handle the `update-filter` event with the `handle_event/3`\ncallback function of your LiveView.\n\n```elixir\n@impl true\ndef handle_event(\"update-filter\", params, socket) do\n  params = Map.delete(params, \"_target\")\n  {:noreply, push_patch(socket, to: ~p\"/pets?#{params}\")}\nend\n```\n\nNote that while the `filter_fields` component produces all necessary hidden\ninputs, it doesn't automatically render inputs for filter values. Instead, it\npasses the necessary details to the inner block, allowing you to customize the\nfilter inputs with your own input component.\n\nYou can pass additional options for each field. Refer to the\n`Flop.Phoenix.filter_fields/1` documentation for details.","ref":"readme.html#filter-forms"},{"type":"extras","title":"Adding visible inputs for meta parameters - Flop Phoenix","doc":"If you want to render visible inputs instead of relying on the hidden inputs\nthat are automatically added to the form, you can just add them to the form\ncomponent:\n\n```heex\n<.form\n  for={@form}\n  id={@id}\n  phx-target={@target}\n  phx-change={@on_change}\n  phx-submit={@on_change}\n>\n  <%!-- ... --%>\n\n   Page size \n   \n\n   reset \n</.form>\n```\n\n`Phoenix.LiveView.JS` command as an attribute to the components in that case.","ref":"readme.html#adding-visible-inputs-for-meta-parameters"},{"type":"extras","title":"LiveView streams - Flop Phoenix","doc":"To use LiveView streams, you can change your `handle_params/3` function as\nfollows:\n\n```elixir\ndef handle_params(params, _, socket) do\n  {:ok, {pets, meta}} = Pets.list_pets(params)\n  {:noreply, socket |> assign(:meta, meta) |> stream(:pets, pets, reset: true)}\nend\n```\n\nWhen using LiveView streams, you need to pass `@streams.pets` instead of `@pets`\nto the table component.\n\nThe stream values are tuples, with the DOM ID as the first element and the items\n(in this case, Pets) as the second element. You need to match on these tuples\nwithin the `:let` attributes of the table component.\n\n```heex\n \n  <:col :let={{_, pet}} label=\"Name\" field={:name}>{pet.name}</:col>\n  <:col :let={{_, pet}} label=\"Age\" field={:age}>{pet.age}</:col>\n \n```","ref":"readme.html#liveview-streams"},{"type":"extras","title":"Customization - Flop Phoenix","doc":"For customizing the components, it is recommend to define wrapper components\nthat set the necessary attributes. Refer to the\n[module documentation](https://hexdocs.pm/flop_phoenix/Flop.Phoenix.html#module-customization) for examples.","ref":"readme.html#customization"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"## [0.25.0] - 2025-05-04","ref":"changelog.html#unreleased"},{"type":"extras","title":"Added - Changelog","doc":"- Add guide about load-more buttons and infinite scroll.\n- Add guide about CSS styles.\n- Add guide about page size controls.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Remove `opts` attribute from `Flop.Phoenix.pagination/1` and remove\n  configuration of the pagination component via application environment.\n- Remove all default classes from pagination component.\n- Remove `wrapper_attrs` from pagination component in favor of global\n  attributes.\n- Set page link aria label function with `page_link_aria_label_fun` attribute\n  on `Flop.Phoenix.pagination/1` component instead of using\n  `pagination_link_aria_label` option.\n- Set previous and next link attributes and content with slots instead of\n  options.\n- Set ellipsis content with slot instead of option.\n- Set pagination list attributes with `page_list_attrs` attribute instead of\n  `pagination_list_attrs` option.\n- Set pagination list item attributes with `page_list_item_attrs` attribute\n  instead of `pagination_list_item_attrs` option.\n- Set pagination link attributes with `page_link_attrs` attribute\n  instead of `pagination_link_attrs` option.\n- Set current pagination link attributes with `current_page_link_attrs`\n  attribute instead of `current_link_attrs` option.\n- Set disabled link attributes with `disabled_link_attrs` attribute instead of\n  `disabled_attrs` option.\n- Remove default classes for pagination root element, pagination list and\n  pagination links.\n- Use ` ` elements for pagination if no `path` is set.\n- Remove `role` attribute from the pagination component. The ` ` element\n  already has the implicit ARIA role `navigation`.\n- Add `rel` attribute to previous/next links.\n- Mark up disabled previous/next links of the pagination component as\n  ` ` instead of ` `.\n- Update documentation for `hidden_inputs_for_filter/1` to use\n  `Phoenix.Component.inputs_for/1` with the `skip_persistent_id` option.\n- Require Phoenix >= 1.6.0 and < 1.9.0.","ref":"changelog.html#changed"},{"type":"extras","title":"How to upgrade - Changelog","doc":"The configuration via `pagination_opts` is deprecated. Instead of referencing a\nfunction that returns the pagination opts, it is recommended to define a\ncomponent that wraps the `Flop.Phoenix.pagination/1` component and sets the\nnecessary attributes.\n\nFollow the upgrade guide of version 0.24.0 to update the pagination component\nconfiguration, including the removal of the application configuration, if you\nhaven't done so already.\n\nRemove the `wrapper_opts` from your pagination options and pass them directly\nas attributes instead.\n\n```diff\n   \n```\n\nReplace the `:previous_link_attrs`, `:previous_link_content`, `next_link_attrs`,\nand `next_link_content` attributes with the `previous` and `next` slots.\n\n```diff\n   )),\n-     next_link_attrs: [class: \"next\"],\n-     next_link_content: Phoenix.HTML.raw(~s( ))\n    ]}\n- />\n+ >\n+   <:previous attrs={[class: \"previous\"]}>\n+      \n+   </:previous>\n+   <:next attrs={[class: \"next\"]}>\n+      \n+   </:next>\n+  \n```\n\nReplace the `:ellipsis_attrs` and `:ellipsis_content` attributes with the\n`ellipsis` slot.\n\n```diff\n   \n+ >\n+   <:ellipsis>\n+      ‥ \n+   </:ellipsis>\n+  \n```\n\nRemove the `:disabled_attrs` option. Note that the `disabled` and\n`aria-disabled` attributes are set automatically and do not need to be passed\nhere.\n\nTo style disabled links/buttons without a CSS class, use the CSS selector\n`[disabled], [aria-disabled=\"true\"]`.\n\n```diff\n   \n```\n\nRemove the `:pagination_link_aria_label` option and set the\n`page_link_aria_label_fun` attribute instead.\n\n```diff\n   \n```\n\nRemove the `:pagination_list_attrs`, `:pagination_list_item_attrs`,\n`:pagination_link_attrs`, and `:current_link_attrs` options and set the\n`page_list_attrs`, `page_list_item_attrs`, `page_link_attrs`, and\n`current_page_link_attrs` attributes instead. If you relied on the default page\nlist class and page link classes, set them explicitly. Note that the\n`aria-current` attribute is set automatically and does not need to be set here.\n\nTo style the current page link without a CSS class, use the CSS selector\n`[aria-current=\"page\"]`.\n\n```diff\n   \n```\n\nEnsure that your CSS styles work with both ` ` elements (used when a `path` is\nset) and ` ` elements (used when no `path` is set). Check the appearance\nof the pagination component carefully.\n\n## [0.24.0] - 2025-02-01","ref":"changelog.html#how-to-upgrade"},{"type":"extras","title":"Added - Changelog","doc":"- Add `Flop.Phoenix.Pagination` struct to hold information needed to render\n  a pagination component.\n- Add `Flop.Phoenix.Pagination.new/2` to build a `Pagination` struct from a\n  `Flop.Meta` struct.\n- Add `Flop.Phoenix.pagination_for/1` for building custom pagination components.\n  Update the existing `Flop.Phoenix.pagination/1` component to use it.\n- Add `Flop.Phoenix.page_link_range/3` for determining which page links to\n  render in a pagination component.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Support cursor pagination in `Flop.Phoenix.pagination/1`.\n- Remove the `page_links` option from the `pagination_opts`. Set this option\n  with the new `page_links` attribute instead.\n- Change the values of the `page_links` option from\n  `:all | :hide | {:ellipsis, pos_integer}` to `:all | :none | pos_integer`.\n- Change default value for `page_links` option of the\n  `Flop.Phoenix.pagination/1` component from `:all` to `5`.\n- Deprecate configuring the `pagination` and `table` components via the\n  application environment. Define wrapper components that pass the `opts`\n  attribute to the Flop.Phoenix components instead.\n- Remove dependency on PhoenixHTMLHelpers.\n- Require Phoenix LiveView ~> 1.0.0.\n- Require Elixir ~> 1.14.","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"- Remove `Flop.Phoenix.cursor_pagination/1`. Use `Flop.Phoenix.pagination/1`\n  instead.\n- Remove `t:Flop.Phoenix.cursor_pagination_option/0`.\n- Remove `Flop.Phoenix.IncorrectPaginationTypeError`.\n- Remove `input_type/3` from the `Phoenix.HTML.FormData` protocol\n  implementation for `Flop.Meta`. The function had been removed from the\n  protocol in Phoenix.HTML 4.0.\n- Remove the previously deprecated `event` attribute from the\n  `Flop.Phoenix.pagination/1` and `Flop.Phoenix.table/1` components. Use\n  `:on_paginate` and `:on_sort` instead.\n- Remove the previously deprecated `hide` and `show` attributes from the\n  `:col` and `:action` slots of the `Flop.Phoenix.table/1` component. Use the\n  `:if` attribute instead.","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix a warning about ranges in Elixir 1.18.","ref":"changelog.html#fixed"},{"type":"extras","title":"How to upgrade - Changelog","doc":"Replace `Flop.Phoenix.cursor_pagination/1` with `Flop.Phoenix.pagination/1`.\n\n```diff\n-  \n+  \n```\n\nUpdate the format of the `page_links` option for the pagination component.\n\n```diff\n- page_links: {:ellipsis, 7},\n+ page_links: 7,\n\n- page_links: :hide,\n+ page_links: :none,\n```\n\nRemove `page_links` from your `pagination_opts` function and add it as an\nattribute instead.\n\n```diff\ndef pagination_opts do\n  [\n    ellipsis_attrs: [class: \"ellipsis\"],\n    ellipsis_content: \"‥\",\n    next_link_attrs: [class: \"next\"],\n    next_link_content: next_icon(),\n-     page_links: 7,\n    pagination_link_aria_label: &\"#{&1}ページ目へ\",\n    previous_link_attrs: [class: \"prev\"],\n    previous_link_content: previous_icon()\n  ]\nend\n\n \n```\n\nReplace the `:show` and `:hide` attribute in the `:col` slot of the table\ncomponent with `:if`.\n\n```diff\n<:col\n  :let={pet}\n- show={@admin?}\n+ :if={@admin?}\n  label=\"Name\"\n  field={:name}\n>\n  <%= pet.name %>\n</:col>\n\n<:col\n  :let={pet}\n- hide={!@admin?}\n+ :if={@admin?}\n  label=\"Name\"\n  field={:name}\n>\n  <%= pet.name %>\n</:col>\n```\n\nReplace the `event` attribute of the pagination table components with\n`on_paginate` and `on_sort`.\n\n```diff\n \n\n \n```\n\nRemove the configuration for the pagination component from `config/config.exs`\nand define a wrapper component in your `CoreComponents` module instead. This\nis optional, but will make future version updates easier.\n\nFor the pagination component:\n\n```diff\n# config/config.exs\n\nconfig :flop_phoenix,\n- pagination: [opts: {MyAppWeb.CoreComponents, :pagination_opts}],\n  table: [opts: {MyAppWeb.CoreComponents, :table_opts}]\n\n# MyAppWeb.CoreComponents\n\n- def pagination_opts do\n-   [\n-     # ...\n-   ]\n- end\n\n+ attr :meta, Flop.Meta, required: true\n+ attr :path, :any, default: nil\n+ attr :on_paginate, JS, default: nil\n+ attr :target, :string, default: nil\n+\n+ def pagination(assigns) do\n+   ~H\\\"\"\"\n+    \n+   \\\"\"\"\n+ end\n```\n\n## [0.23.1] - 2024-10-17","ref":"changelog.html#how-to-upgrade"},{"type":"extras","title":"Changed - Changelog","doc":"- Raise an error if a meta struct with the wrong pagination type is passed to\n  the `pagination` or `cursor_pagination` component.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix compilation error in Phoenix LiveView 1.0.0-rc.7.\n- Fix type of `row_click` attribute.\n\n## [0.23.0] - 2024-08-18","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Support and require `live_view ~> 1.0.0-rc.0`.\n- Allow to pass options directly in config file instead of referencing function.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed a deprecation warning in Elixir 1.17.\n\n## [0.22.10] - 2024-08-18","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Loosen version requirement for `flop` to support 0.26.\n\n## [0.22.9] - 2024-05-04","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Added `:pagination_list_item_attrs` option to `Flop.Phoenix.pagination/1`.\n\n## [0.22.8] - 2024-03-23","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Support `:col_class` attr on `:col` and `:action` slots in addition to\n  `:col_style`.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Don't render empty `style` attributes on `col` elements in `colgroup`.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- The page range calculation in the `Flop.Phoenix.pagination/1` was incorrect\n  towards the last pages.\n\n## [0.22.7] - 2024-03-02","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Loosen version requirement for `phoenix_html`.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Warning when wrapping table component and passing on `:col` slot as attribute.\n\n## [0.22.6] - 2024-01-14","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Support Flop 0.25.0.\n- Update documentation examples for filter forms.\n\n## [0.22.5] - 2023-12-24","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Requires `phoenix_html ~> 4.0`.\n\n## [0.22.4] - 2023-11-18","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Don't render `li` element if a pagination link is not rendered.\n\n## [0.22.3] - 2023-11-14","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Support Flop ~> 0.24.0.\n\n## [0.22.2] - 2023-10-19","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Numbered pagination links were not wrapped in `li` elements.\n\n## [0.22.1] - 2023-09-28","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Allow to use `t:Phoenix.HTML.safe/0` as a label attribute in table headers.\n\n## [0.22.0] - 2023-09-26","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Added `directions` attribute to the `col` slot of the table component. This\n  allows you to override the default sort directions, e.g. to specify\n  nulls-first or nulls-last.\n- Added `thead_th_attrs` and `th_wrapper_attrs` attributes to the `col` slot\n  of the table component.\n- Added `thead_th_attrs` attribute to the `action` slot of the table component.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Renamed `attrs` attribute on the `col` and `action` slots of the table\n  component to `tbody_td_attrs` in order to match the naming of the global\n  table options.","ref":"changelog.html#changed"},{"type":"extras","title":"How to upgrade - Changelog","doc":"Rename the `attrs` attribute to `tbody_td_attrs` in both the `col` slot and the\n`action` slot:\n\n```diff\n \n-  <:col :let={p} attrs={[class=\"my-class\"]}><%= p.id %></:col>\n-  <:action :let={p} attrs={[class=\"my-class\"]}>button</:col>\n+  <:col :let={p} tbody_td_attrs={[class=\"my-class\"]}><%= p.id %></:col>\n+  <:action :let={p} tbody_td_attrs={[class=\"my-class\"]}>button</:col>\n \n```\n\n## [0.21.2] - 2023-09-26","ref":"changelog.html#how-to-upgrade"},{"type":"extras","title":"Changed - Changelog","doc":"- Support `phoenix_liveview ~> 0.20`.\n\n## [0.21.1] - 2023-08-05","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Allow passing a function to the `attrs` option of table component's `:action`\n  slot. Before, this was only supported in the `:col` slot.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Improve some error messages and documentation examples.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- In the `:col` and `:action` slots of the table component, the `attrs` option\n  did not properly override the attributes set with the `:tbody_td_attrs`\n  option.\n\n## [0.21.0] - 2023-07-17","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Depend on flop ~> 0.22.0.\n\n## [0.20.0] - 2023-07-09","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Added an `on_paginate` attribute to the `pagination` and `cursor_pagination`\n  components. This attribute takes a `Phoenix.LiveView.JS` command as a value.\n  The attribute can be combined with the `path` attribute, in which case the URL\n  will be patched _and_ the the given JS command is executed.\n- Similarly, an `on_sort` attribute was added to the `table` component.\n- Allow setting `tbody_tr_attrs` to a function that takes the current row item\n  as an argument and returns a list of attributes.\n- Allow setting the `attrs` attribute of the `:col` slot of the table component\n  to a function that takes the current row item as an argument and returns a\n  list of attributes.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- The table ID falls back to `\"sortable-table\"` if no schema module is used.\n- The tbody ID was changed to `{id}-tbody`.\n- The table container ID is set to `{id}-container` now.\n- The table ID is set to `{id}` now.","ref":"changelog.html#changed"},{"type":"extras","title":"Deprecated - Changelog","doc":"- The `show` and `hide` attributes of the `:col` slot of the table component are\n  now deprecated in favor of the `:if` attribute.\n- The `event` attribute of the pagination, cursor pagination and table\n  components has been deprecated in favor of `on_pagination` and `on_sort`.\n\n## [0.19.1] - 2023-06-30","ref":"changelog.html#deprecated"},{"type":"extras","title":"Changed - Changelog","doc":"- The table component only renders an ascending/descending order indicator in\n  the column header for the first order field, instead rendering one in the\n  column headers for all ordered fields.\n- Support Flop 0.22.\n\n## [0.19.0] - 2023-05-30","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Necessary updates for `phoenix_live_view ~> 0.19.0`.\n- Requires `phoenix_live_view ~> 0.19.0`.\n- Remove previously deprecated `Flop.Phoenix.pop_filter/2`. Use\n  `Flop.Filter.pop/3` instead.\n\n## [0.18.2] - 2023-05-08","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Added `hidden` attribute to hidden inputs rendered by `filter_fields`\n  component in order to solve CSS spacing issues.\n\n## [0.18.1] - 2023-03-18","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Added support for LiveView streams to the table component. To this end,\n  `row_id`, `row_item` and `id` attributes were added to the component,\n  following the example of Phoenix 1.7. The `id` attribute is added to the\n  `tbody`. If no `id` is explicitly set, a default value will be used depending\n  on the schema name.\n\n## [0.18.0] - 2023-02-25","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Added dependency on `Phoenix.HTML` `~> 3.3.0`.\n- The `filter_fields` component now passes the `Phoenix.HTML.FormField` struct\n  introduced in `Phoenix.HTML` 3.3.0 to the inner block.\n- Support `:as` option for filter inputs with `Phoenix.HTML.FormData/4`\n  (`inputs_for`).\n- `Phoenix.HTML.FormData.input_type/2` now considers the Ecto type for join,\n  custom and compound fields.\n- Remove support for `path_helper` assigns, previously deprecated in 0.15.\n- Deprecate `Flop.Phoenix.pop_filter/2`.","ref":"changelog.html#changed"},{"type":"extras","title":"How to upgrade - Changelog","doc":"If your `input` component already knows how to handle the\n`Phoenix.HTML.FormField` struct, you can update the inner block for\n`filter_fields` like this:\n\n```diff\n<.filter_fields :let={i} form={f} fields={[:email, :name]}>\n  <.input\n-    field={{i.form, i.field}}\n+    field={i.field}\n    type={i.type}\n    label={i.label}\n-    id={i.id}\n-    name={i.name}\n-    value={i.value}\n    {i.rest}\n  />\n</.filter_fields>\n```\n\nIf your `input` component still expects the individual assigns, you can update\nthe inner block like this:\n\n```diff\n<.filter_fields :let={i} form={f} fields={[:email, :name]}>\n  <.input\n-    field={{i.form, i.field}}\n+    field={{i.field.form, i.field.field}}\n    type={i.type}\n    label={i.label}\n-    id={i.id}\n-    name={i.name}\n-    value={i.value}\n+    id={i.field.id}\n+    name={i.field.name}\n+    value={i.field.value}\n    {i.rest}\n  />\n</.filter_fields>\n```\n\nFor an upgrade example for the `path_helper` assign, see the changelog entry for\nversion 0.15.0.\n\n## [0.17.2] - 2023-01-15","ref":"changelog.html#how-to-upgrade"},{"type":"extras","title":"Changed - Changelog","doc":"- Support Flop 0.19.\n\n## [0.17.1] - 2022-11-15","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Allow passing an `offset` when generating filter inputs with\n  `Phoenix.HTML.Form.inputs_for/3`.\n\n## [0.17.0] - 2022-10-27","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- New option `tbody_attrs` for table component.\n- New attribute `row_click` and slot `action` for table component.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- To pass additional attributes to a column, you will now have to use the\n  `attrs` attribute. This was necessary because defining a `global` attribute on\n  a slot causes a compile-time error in `phoenix_live_view` 0.18.3.\n\n```diff\n \n-  <:col :let={p} class=\"my-class\"><%= p.id %></:col>\n+  <:col :let={p} attrs={[class=\"my-class\"]}><%= p.id %></:col>\n \n```\n\n## [0.16.0] - 2022-10-10","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- New Phoenix component `Flop.Phoenix.hidden_inputs_for_filter/1`.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Major refactoring of `Flop.Phoenix.filter_fields/1`. Instead of giving you the\n  rendered ` ` and ` ` elements, the component now only passes the\n  necessary arguments to the inner block. You will have to pass these arguments\n  to your own `input` component (or whatever you name it). The field option\n  format has also been updated. These changes were made to fix warnings emitted\n  by live view 0.18.1, and also accompany current changes in Phoenix that thin\n  out `Phoenix.HTML` / `Phoenix.HTML.Form` in favor of Phoenix components for\n  inputs.\n- Require `flop >= 0.17.1 and < 0.19.0`.","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"- Removed `Flop.Phoenix.filter_hidden_inputs_for/1`. This function is not used\n  internally anymore. You can either use `Phoenix.HTML.Form.hidden_inputs_for/1`\n  (Phoenix.HTML ~> 3.2), or use `Flop.Phoenix.hidden_inputs_for_filter/1`,\n  which does the same, but as a Phoenix component.\n- Removed `Flop.Phoenix.filter_label/1` and `Flop.Phoenix.filter_input/1`. With\n  the changes to `Flop.Phoenix.filter_fields/1` and the move away from the\n  input rendering functions of `Phoenix.HTML.Form`, these functions don't have\n  any value anymore. Read the documentation of\n  `Flop.Phoenix.hidden_inputs_for_filter/1` for an example on how to easily\n  render the fields of individual filters.","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed warnings about tainted variables in live view 0.18.1.\n- Fixed an issue where default parameters set in the backend module were not\n  removed from the query parameters.\n- Fixed URLs ending with `?` when no query parameters are necessary if the path\n  is passed as a string.","ref":"changelog.html#fixed"},{"type":"extras","title":"How to upgrade - Changelog","doc":"#### Filter fields component\n\nPreviously, you would render a filter form like this:\n\n```heex\n<.form :let={f} for={@meta}>\n   \n     \n      <%= entry.label %>\n      <%= entry.input %>\n     \n   \n</.form>\n```\n\nIn this example, `entry.label` and `entry.input` are complete ` ` and\n` ` elements with all attributes set. You will need to change this to:\n\n```heex\n<.form :let={f} for={@meta}>\n  <.filter_fields :let={i} form={f} fields={[:name, :email]}>\n    <.input\n      id={i.id}\n      name={i.name}\n      label={i.label}\n      type={i.type}\n      value={i.value}\n      field={{i.form, i.field}}\n      {i.rest}\n    />\n  </.filter_fields>\n</.form>\n```\n\nYou will have to define an `input` component in your project. You can take a\nhint from the `input` component that is generated as part of the [Components\nmodule](https://github.com/phoenixframework/phoenix/blob/master/priv/templates/phx.gen.live/components.ex) by Phoenix 1.7.\n\n#### Field options\n\nRemove `input_opts` and `label_opts` and pass them directly to your `input`\ncomponent, or add them directly to the `input` component. If you passed an `id`\nto `filter_fields`, set in on the `form` instead.\n\n```diff\n<.filter_fields\n  :let={i}\n  form={f}\n  fields={[:name]}\n-  id=\"some-id\"\n-  input_opts={[class: \"input\", phx_debounce: 100]}\n-  label_opts={[class: \"label\"]}\n>\n  <.input\n     ...\n+    class=\"input\"\n+    phx-debounce={100}\n  />\n</.filter_fields>\n```\n\nUse strings instead of atoms to set the type, and use the types that your\n`input` component understands.\n\n```diff\n<.filter_fields\n  :let={i}\n  form={f}\n  fields={[\n-    name: [type: :text_input],\n+    name: [type: \"text\"],\n-    age: [type: :number_input],\n+    age: [type: \"number\"],\n-    phone: [type: :telephone_input],\n+    phone: [type: \"tel\"]\n  ]}\n>\n```\n\nIf you passed additional input function options in a tuple, take them out of\nthe tuple and add them to the keyword list instead.\n\n```diff\n<.filter_fields\n  :let={i}\n  form={f}\n  fields={[\n-    role: [type: {:select, [\"author\", \"editor\"], class: \"select\"}]\n+    role: [type: \"select\", options: [\"author\", \"editor\"], class: \"select\"]\n  ]}\n>\n```\n\nThe `default` option is not handled for you anymore. You can still set it, but\nit will just be passed on as part of the `rest` options, so your `input`\ncomponent will need to handle it.\n\n#### Filter label and input components\n\nIf you used `Flop.Phoenix.filter_label/1` or `Flop.Phoenix.filter_input/1`\nbefore, follow the example in the documentation of\n`Flop.hidden_inputs_for_filter/1` to render the inputs of individual filters\nwithout the removed components.\n\n## [0.15.2] - 2022-10-10","ref":"changelog.html#how-to-upgrade"},{"type":"extras","title":"Changed - Changelog","doc":"- Change version requirement for Flop to `>= 0.15.0 and < 0.19.0`.\n\n## [0.15.1] - 2022-09-30","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Typespec of `Flop.Phoenix.build_path/3`.\n\n## [0.15.0] - 2022-09-22","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"This release adds support for passing URI strings instead of MFA or FA tuples to\ncomponents and helper functions. This allows you to use the library with the\nverified routes introduced in Phoenix 1.7. Alternatively, you can now also\nchoose to pass a 1-ary path builder function. See `Flop.Phoenix.build_path/3`\nfor examples. Passing tuples pointing to route helpers is still supported.\n\n- Added an example for a custom filter form component to the readme.\n- Support passing a URI string as a path to the `table`, `pagination` and\n  `cursor_pagination` components and to `build_path/3`.\n- Support passing a 1-ary path builder function to the `table`, `pagination` and\n  `cursor_pagination` components and to `build_path/3`.\n- New function `Flop.Phoenix.pop_filter/2`.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Require `live_view ~> 0.18.0`.\n- Deprecate `path_helper` assign in favor of `path`.\n- Use declarative assigns and replace `Phoenix.LiveView.Helpers.live_patch/1`\n  with `Phoenix.Component.link/1`.\n- `Flop.Phoenix.filter_input/1` requires additional options for the input\n  function to be passed in the `input_opts` assign, instead of passing them\n  directly to the component. This was necessary because the global attributes\n  you can define with declarative assigns in LiveView 0.18 are meant for HTML\n  attributes, while the input options may contain any additional attributes\n  necessary (e.g. a list of select options that are rendered as option\n  elements).","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Apply `show` and `hide` attribute for columns to `colgroup` as well.\n- Correctly handle multiple inputs for the same field in `Flop.filter_fields/1`.","ref":"changelog.html#fixed"},{"type":"extras","title":"How to upgrade - Changelog","doc":"Rename the `path_helper` assigns of `table`, `pagination` and\n`cursor_pagination` components to `path`.\n\n```diff\n- <.pagination meta={@meta} path_helper={{Routes, :pet_path, [@socket, :index]}} />\n+ <.pagination meta={@meta} path={{Routes, :pet_path, [@socket, :index]}} />\n```\n\nWrap additional options passed to `Flop.Phoenix.filter_input/1` into a single\n`input_opts` assign.\n\n```diff\n- <.filter_input form={ff} class=\"select\" options={[:some, :options]} />\n+ <.filter_input form={ff} input_opts={[class: \"select\", options: [:some, :options]]} />\n```\n\n## [0.14.2] - 2022-08-26","ref":"changelog.html#how-to-upgrade"},{"type":"extras","title":"Changed - Changelog","doc":"- Support Flop `~> 0.17.0`.\n\n## [0.14.1] - 2022-03-22","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Support Flop `~> 0.16.0`.\n\n## [0.14.0] - 2022-02-22","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- `symbol_unsorted` option for the `table` component.\n- `caption` assign for the `table` component.\n- `col_style` assign for the `:col` slot of the `table` component.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Additional attributes passed to the `<:col>` slot will now be added as\n  attributes to the ` ` tags.\n\n## [0.13.0] - 2021-11-14","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `cursor_pagination/1` component.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- The pagination component adds the `disabled` class to the `span` that is\n  displayed when the previous or next link is disabled now. Previously, the\n  `disabled` attribute was set on the `span`. The class can be customized with\n  the `:disabled_class` option.\n\n## [0.12.0] - 2021-11-08","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Implement the `Phoenix.HTML.FormData` protocol for `Flop.Meta`. This means\n  you can pass the meta struct as `:for` option to the Phoenix `form_for`\n  functions now.\n- Add the functions `filter_fields/1`, `filter_input/1` and `filter_label/1`.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Remove `:for` option. The schema module is now automatically derived from the\n  meta struct.\n\n## [0.11.1] - 2021-10-31","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Adds `hide` and `show` options to table `:col`.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Passing a `label` to a table `:col` is now optional.\n\n## [0.11.0] - 2021-10-30","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- The `path_helper_args` assign has been removed in favor of passing mfa\n  tuples as `path_helper`.\n- In the same vein, `Flop.Phoenix.build_path/4` has been replaced with\n  `Flop.Phoenix.build_path/3`, which also takes a tuple as the first argument.\n- The table component has been changed to use slots. The `headers`,\n  `footer`, `row_func` and `row_opts` assigns have been removed. Also, the\n  `tfoot_td_attrs` and `tfoot_th_attrs` options have been removed.\n- The `live_view` version requirement has been changed to `~> 0.17.0`.\n- Better error messages for invalid assigns have been added.","ref":"changelog.html#changed"},{"type":"extras","title":"How to upgrade - Changelog","doc":"Update the `path_helper` and `path_helper_args` assigns set for the `table`\nand `pagination` component:\n\n```diff\n- path_helper={&Routes.pet_path/3}\n- path_helper_args={[@socket, :index]}\n+ path_helper={{Routes, :pet_path, [@socket, :index]}}\n```\n\nIf you prefer, you can pass a function instead.\n\n```diff\n+ path_helper={{&Routes.pet_path/3, [@socket, :index]}}\n```\n\nUpdate any calls to `Flop.Phoenix.build_path/4`:\n\n```diff\n- Flop.Phoenix.build_path(&Routes.pet_path/3, [@socket, :index], meta)\n+ Flop.Phoenix.build_path({Routes, :pet_path, [@socket, :index]}, meta)\n```\n\nIf you prefer, you can use a 2-tuple here as well:\n\n```diff\n+ Flop.Phoenix.build_path({&Routes.pet_path/3, [@socket, :index]}, meta)\n```\n\nFinally, update the tables in your templates:\n\n```diff\n  [pet.name, pet.age] end}\n-   footer={[\"\", @average_age]}\n- />\n+ >\n+   <:col let={pet} label=\"Name\" field={:name}><%= pet.name %></:col>\n+   <:col let={pet} label=\"Age\" field={:age}><%= pet.age %></:col>\n\n+   <:foot>\n+      \n+         \n+        <%= @average_age %> \n+      \n+   </:foot>\n+  \n```\n\nAlso, you can remove `tfoot_td_attrs` and `tfoot_th_attrs` from the `opts`\nassign (or opts provider function).\n\n## [0.10.0] - 2021-10-24","ref":"changelog.html#how-to-upgrade"},{"type":"extras","title":"Added - Changelog","doc":"- It is now possible to set global options for the components in your config.\n\n```elixir\nconfig :flop_phoenix,\n  pagination: [opts: {MyApp.ViewHelpers, :pagination_opts}],\n  table: [opts: {MyApp.ViewHelpers, :table_opts}]\n```","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- The `for`, `event` and `target` options moved from the `opts` assign to the\n  root. The `opts` assign is now exclusively used for customization options\n  that modify the appearance, which are usually set globally for a\n  project and are not related to the specific data or view.\n- The `row_func/2` function passed to the `table` component receives the new\n  `row_opts` assign now instead of the `opts` assign.\n- The pagination and table components only pass the `for` option to the query\n  builder, instead of all `opts`.\n- The `path_helper` and `path_helper_args` assigns are now optional if an\n  `event` is passed. A descriptive error is raised if neither of them are\n  passed.\n- The `opts` assign for the pagination and table components is now optional.\n- Aria labels were added to the links to the first and last page.\n- The `aria-sort` attribute was added to the table headers.","ref":"changelog.html#changed"},{"type":"extras","title":"How to upgrade - Changelog","doc":"1. Remove the `for`, `event` and `target` from the `opts` assign and add them\n   as regular assigns at the root level.\n2. Move any key/value pairs that are needed by your `row_func` from `opts` to\n   `row_opts`.\n\nFor example, if your `row_func` looks like this:\n\n```elixir\ndef table_row(%Pet{id: id, name: name}, opts) do\n  socket = Keyword.fetch!(opts, :socket)\n  [name, link(\"show\", to: Routes.pet_path(socket, :show, id))]\nend\n```\n\nUpdate your template like this:\n\n```diff\n \n\n \n```\n\n## [0.9.1] - 2021-10-22","ref":"changelog.html#how-to-upgrade"},{"type":"extras","title":"Changed - Changelog","doc":"- Change `live_view` version requirement to `~> 0.16.0 or ~> 0.17.0`.\n\n## [0.9.0] - 2021-10-04","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add table foot option for sortable table.\n\n## [0.8.1] - 2021-08-11","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Loosen version requirement for Flop.\n\n## [0.8.0] - 2021-08-11","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- New options `event` and `target` for the pagination and sortable table\n  component, which allow to emit pagination and sorting events in LiveView\n  without patching the URL.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Use `HEEx` templates for both the pagination and the sortable table component.\n  Refer to the Readme for usage examples.\n- Require `live_view ~> 0.16.0`.\n- Support safe HTML tuples in unsortable table headers.\n- Improve documentation with examples for LiveView, HEEx templates and EEx\n  templates.\n\n## [0.7.0] - 2021-06-13","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add wrapper around sortable table header link and order direction indicator.\n- Add option `current_link_attrs` to pagination builder.\n- Add options `thead_tr_attrs`, `thead_th_attrs`, `tbody_tr_attrs` and\n  `tbody_td_attrs` to table generator.\n- Add option `no_results_content` to table generator, which sets the content\n  that is going to be displayed instead of the table if the item list is empty.\n  A default option is applied, so make sure to set the option and/or remove your\n  own no results messages from your templates when making the upgrade.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- The table options `table_class`, `th_wrapper_class`, `symbol_class` and\n  `container_class` were replaced in favour of `table_attrs`,\n  `th_wrapper_attrs`, `symbol_attrs` and `container_attrs` for more flexibility\n  and consistency with the pagination generator. To update, rename the options\n  in your code and wrap the values in keyword lists with a `class` key\n  (e.g. `container_class: \"table-container\"` =>\n  `container_attrs: [class: \"table-container\"]`).\n- The `pagination_link_attrs` is not applied to current links anymore. Use\n  `current_link_attrs` to set the attributes for the current link.\n- Omit `page=1` and `offset=0` when building query parameters.\n- Omit default values for order and limit/page size parameters when building\n  query parameters.\n- Requires Flop `~> 0.11.0`.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Order direction indicator was wrapped twice.\n- A Flop struct with an offset was resulting in invalid pagination links.\n\n## [0.6.1] - 2021-05-05","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Pagination helper generated invalid links when using `default_limit` option.\n\n## [0.6.0] - 2021-05-04","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `Flop.Phoenix.table/1` for rendering sortable tables.\n- Add function `Flop.Phoenix.to_query/1`, which converts a Flop struct into\n  a keyword list for query parameters.\n- Add function `Flop.Phoenix.build_path/3`, which applies Flop parameters to a\n  Phoenix path helper function.","ref":"changelog.html#added"},{"type":"extras","title":"Removed - Changelog","doc":"- Remove `Flop.Phoenix.Live.PaginationComponent` in favor of making\n  `Flop.Phoenix.pagination/4` work in both `.eex` and `.leex` templates.\n\n## [0.5.1] - 2021-04-14","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Merge pagination query parameters into existing query parameters, if present.\n\n## [0.5.0] - 2021-03-23","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Rename `FlopPhoenix` to `Flop.Phoenix`.\n- Add `Flop.Phoenix.Live.PaginationComponent` for use with `Phoenix.LiveView`.\n- Change pagination to always display links to the first and last page.\n\n## [0.4.0] - 2020-09-04","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Allow usage with newer versions of Flop.\n\n## [0.3.0] - 2020-06-17","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- New option to hide the number of page links.\n- New option to limit the number of page links.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Add order and filter parameters to pagination links.\n\n## [0.2.0] - 2020-06-15","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Improve documentation.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `previous_link/3`, `next_link/3` and `page_links/3` are private functions now.\n\n## [0.1.0] - 2020-06-15\n\nInitial release","ref":"changelog.html#changed"},{"type":"extras","title":"CSS Styles","doc":"# CSS Styles","ref":"css_styles.html"},{"type":"extras","title":"Pagination - CSS Styles","doc":"The pagination component doesn't set any default styles. While you can set class\nattributes for every element, it is entirely possible to style the component\nwith just a base class.\n\n```heex\n \n  <%!-- ... --%>\n \n```\n\nThe class above will be set on the ` ` element that wraps the pagination\ncontrols. With this in place, we can apply some basic styles. For readability,\nwe're going to use nested CSS rules in this example.\n\n```css\n.pagination {\n  display: flex;\n  gap: 0.25rem;\n  align-items: center;\n  justify-content: space-between;\n\n  > ul {\n    display: flex;\n    flex: 1 1 auto;\n    flex-wrap: wrap;\n    gap: 0.25rem;\n    align-items: center;\n    justify-content: flex-end;\n    padding: 0;\n    margin: 0;\n    list-style-type: none;\n\n    li {\n      flex: 0 1 auto;\n      padding: 0;\n      margin: 0;\n    }\n  }\n\n  > a:first-child,\n  > button:first-child {\n    flex: none;\n  }\n\n  > a:last-of-type,\n  > button:last-of-type {\n    flex: none;\n  }\n\n  a,\n  button,\n  li > span {\n    padding: 0.25rem 0.5rem;\n    margin: 0;\n    border-radius: 4px;\n  }\n\n  li > span {\n    color: #00749d;\n  }\n\n  a,\n  button {\n    color: #fff;\n    white-space: nowrap;\n    background: #00749d;\n    border: none;\n\n    &:hover,\n    &:focus {\n      background-color: #003c54;\n    }\n\n    &[aria-current=\"page\"] {\n      background-color: #001d2a;\n    }\n  }\n\n  [disabled],\n  [aria-disabled=\"true\"] {\n    pointer-events: none;\n    opacity: 0.7;\n  }\n}\n```\n\nLet's break this down.\n\n```css\n.pagination {\n  display: flex;\n  gap: 0.25rem;\n  align-items: center;\n  justify-content: space-between;\n\n  // ...\n}\n```\n\nThis selects the ` ` element by using the base class we passed to the\ncomponent as an attribute.\n\n```css\n> ul {\n  display: flex;\n  flex: 1 1 auto;\n  flex-wrap: wrap;\n  gap: 0.25rem;\n  align-items: center;\n  justify-content: flex-end;\n  padding: 0;\n  margin: 0;\n  list-style-type: none;\n\n  li {\n    flex: 0 1 auto;\n    padding: 0;\n    margin: 0;\n  }\n}\n```\n\nThe numbered page links are rendered within an unordered list, which is a\ndirect descendant of the navigation element.\n\n```css\n> a:first-child,\n> button:first-child {\n  flex: none;\n}\n\n> a:last-of-type,\n> button:last-of-type {\n  flex: none;\n}\n```\n\nThese are styles for the previous link or button (first child) and the next\nlink or button (last of type). Depending on whether the `path` attribute is\nset on the component, Flop.Phoenix will either use ` ` elements with an `href`\nor ` ` elements to adhere to proper semantics. We need to account for\nthis in our styles.\n\n```css\na,\nbutton,\nli > span {\n  padding: 0.25rem 0.5rem;\n  margin: 0;\n  border-radius: 4px;\n}\n\nli > span {\n  color: #00749d;\n}\n```\n\nHere we set some basic styles for the links/buttons (both previous/next and\nnumbered page links). The `li > span` selector is used to style the ellipsis,\nwhich uses a classless `span` by default.\n\n```css\na,\nbutton {\n  color: #fff;\n  white-space: nowrap;\n  background: #00749d;\n  border: none;\n\n  &:hover,\n  &:focus {\n    background-color: #003c54;\n  }\n\n  &[aria-current=\"page\"] {\n    background-color: #001d2a;\n  }\n}\n```\n\nHere we set some more styles on the links/buttons, without the ellipsis span.\nThe pagination component sets the `aria-current` attribute on the page number\nlink/button to the current page. We can bind our styles to this attributes\nwithout needing a class.\n\n```css\n[disabled],\n[aria-disabled=\"true\"] {\n  pointer-events: none;\n  opacity: 0.7;\n}\n```\n\nFinally, we define styles for disabled previous/next links and buttons. If\nbuttons are used, the `disabled` attribute is set. Since the `disabled`\nattribute cannot be used with links, the `aria-disabled` attribute is set on\n` ` elements.","ref":"css_styles.html#pagination"},{"type":"extras","title":"Changing the order - CSS Styles","doc":"The pagination component will always render the same markup: Individual\nprevious/next links/buttons first, and then a list with page number\nlinks/buttons. To visually change the order while maintaining the same markup,\nyou can use the `order` attribute.\n\nTo put the page number links/buttons first and the previous/next links/buttons\nlast:\n\n```css\n> a:first-child,\n> button:first-child {\n  order: 2;\n}\n\n> a:last-of-type,\n> button:last-of-type {\n  order: 3;\n}\n\n> ul {\n  order: 1;\n}\n```\n\nTo put the page number links/buttons in the middle between the previous\nlink/button and the next link/button:\n\n```css\n> a:first-child,\n> button:first-child {\n  order: 1;\n}\n\n> a:last-of-type,\n> button:last-of-type {\n  order: 3;\n}\n\n> ul {\n  order: 2;\n}\n```\n\nAdjust the remaining flex properties as needed.","ref":"css_styles.html#changing-the-order"},{"type":"extras","title":"Load More and Infinite Scroll","doc":"# Load More and Infinite Scroll\n\nYou can use Flop's cursor pagination to implement load-more buttons and infinite\nscroll.\n\nLet's start with a basic \"Load More\" button.","ref":"load_more_and_infinite_scroll.html"},{"type":"extras","title":"Load More Button - Load More and Infinite Scroll","doc":"We'll define a basic schema:\n\n```elixir\ndefmodule Petshop.Pets.Pet do\n  use Ecto.Schema\n  import Ecto.Changeset\n\n  @derive {Flop.Schema,\n           filterable: [:name],\n           sortable: [:id],\n           default_order: %{order_by: [:id], order_directions: [:desc]},\n           default_limit: 50}\n\n  schema \"pets\" do\n    field :name, :string\n\n    timestamps(type: :utc_datetime)\n  end\nend\n```\n\nWe derive the `Flop.Schema` protocol, make the `id` column sortable, set the\ndefault order to `id DESC`, and set the default limit to 50.\n\nThen we define a list function:\n\n```elixir\ndef list_pets(params) do\n  Flop.validate_and_run!(Pet, params,\n    for: Pet,\n    default_pagination_type: :first,\n    pagination_types: [:first],\n    replace_invalid_params: true,\n    filtering: false,\n    ordering: false\n  )\nend\n```\n\nWe set the default pagination type to `:first`, so that we get a response with\ncursors even if no pagination parameters were passed. We chose `:first` because\nthe default order in the schema already sorts by ID descending. We also enable\n`replace_invalid_params`, so that invalid parameters are silently ignored, and\nwe disable ordering and filtering via parameters.\n\nIn the `handle_params` function of our LiveView, we simply call the list\nfunction, assign the `meta` struct, and stream the pets.\n\n```elixir\n@impl true\ndef handle_params(params, _url, socket) do\n  {pets, meta} = Pets.list_pets(params)\n  {:noreply, socket |> stream(:pets, pets) |> assign(:meta, meta)}\nend\n```\n\nIn our HEEx template, we're going to use a basic Flop table to render the\npets (but you could use any other component here), and we render the\n\"Load More\" link. We only render the link after checking whether there is\nindeed a next page.\n\n```heex\n  JS.navigate(~p\"/pets/#{pet}\") end}\n>\n  <:col :let={{_id, pet}} label=\"Name\">{pet.name}</:col>\n  <:action :let={{_id, pet}}>\n     \n      <.link navigate={~p\"/pets/#{pet}\"}>Show</.link>\n     \n  </:action>\n \n\n \n  <.link patch={~p\"/pets?after=#{@meta.end_cursor}\"}>Load More</.link>\n \n```\n\nThe \"Load More\" link just attaches the end cursor as `after` parameter to the\nbase URL. A click then triggers the `handle_params` function, which passes the\nparameter on to the list function. And since the end cursor is in the URL, the\nuser can reload the page or the LiveView can be re-mounted without the user\nlosing their position in the list.\n\nAnd that's it - you should now have a working \"Load More\" button.","ref":"load_more_and_infinite_scroll.html#load-more-button"},{"type":"extras","title":"Infinite Scroll - Load More and Infinite Scroll","doc":"Now let's add infinite scroll to the view. First, we'll add a new hook to our\n`assets/app.js`:\n\n```js\nlet Hooks = {};\n\nHooks.InfiniteScroll = {\n  mounted() {\n    observer = new IntersectionObserver(\n      (entries) => {\n        entries.forEach((entry) => {\n          if (entry.isIntersecting) {\n            this.pushEvent(\"load-more\", {});\n          }\n        });\n      },\n      {\n        root: null,\n        rootMargin: \"0px\",\n        threshold: 1.0,\n      },\n    );\n\n    const anchorId = this.el.dataset.anchorId;\n    const anchor = document.getElementById(anchorId);\n\n    if (anchor) {\n      observer.observe(anchor);\n    } else {\n      console.error(`Anchor element not found: ${anchorId}`);\n    }\n  },\n};\n```\n\nWe're using an `IntersectionObserver` to observe an anchor that is referenced with\na data attribute (`data-anchor-id`) on the hook element.\n\nWe then ensure to pass the hook when we create the LiveSocket:\n\n```js\nlet liveSocket = new LiveSocket(\"/live\", Socket, {\n  longPollFallbackMs: 2500,\n  params: { _csrf_token: csrfToken },\n  hooks: Hooks,\n});\n```\n\nAnd that's it on the JavaScript side. In our HEEx template, we wrap our table\nand \"Load More\" link into a `div`, to which we add the `phx-hook` attribute.\n\n```heex\n \n   \n    <%!-- etc. --%>\n   \n\n   \n    <.link patch={~p\"/pets?after=#{@meta.end_cursor}\"}>Load More</.link>\n   \n \n```\n\nNote that the paragraph that contains the \"Load More\" link acts as an anchor: We\ngave it an ID and referenced it in the `data-anchor-id` attribute of the `div`.\n\nWe need one final change: The hook doesn't patch the URL directly, but pushes a\n`load-more` event to the server. If we wanted to patch the URL directly with the\nhook, we'd need to tell it the base URL and the end cursor. While possible, it's\nprobably easier to handle this on the server-side.\n\nTo handle the event, we need to add a `handle_event` function to our LiveView.\nIt checks whether there is indeed an end cursor and patches the URL just like\na click on the \"Load More\" link would do.\n\n```elixir\ndef handle_event(\"load-more\", _, socket) do\n  if end_cursor = socket.assigns.meta.end_cursor do\n    {:noreply, push_patch(socket, to: ~p\"/pets?after=#{end_cursor}\")}\n  else\n    {:noreply, socket}\n  end\nend\n```\n\nSince the paragraph with the \"Load More\" link is only shown if there are more\nitems to show, there won't be any unnecessary events when we reach the end of\nthe list. The visible and focusable \"Load More\" button serves as a fallback for\nusers who rely on keyboard navigation or assistive technologies.","ref":"load_more_and_infinite_scroll.html#infinite-scroll"},{"type":"extras","title":"Final Thoughts - Load More and Infinite Scroll","doc":"This guide only described a basic implementation of infinite scroll using Flop.\nSome details have been omitted, for example:\n\n- If you reload the page with the `after` parameter set, the list will start\n  where you left off, but you cannot view the previous items anymore. You might\n  want to add another \"Load More\" button at the top of the list.\n- The container should have the\n  [\"feed\" role](https://developer.mozilla.org/en-US/docs/Web/Accessibility/ARIA/Reference/Roles/feed_role), which also has recommended [keyboard interactions](https://developer.mozilla.org/en-US/docs/Web/Accessibility/ARIA/Reference/Roles/feed_role#keyboard_interactions).\n- You might want to support the browser's back button.\n- You might want to give users the option to disable infinite scrolling.\n\nApart from the technical side, infinite scroll has a lot of accessibility and\nusability issues. It can interfere with assistive technologies, make it\nharder to navigate or return to a specific spot, prevent a sense of\nprogress or place, and reduce user agency.\n\nBefore you use infinite scroll, consider whether it's truly the best fit for\nyour users. In many cases, conventional pagination or the use of a \"Load More\"\nbutton without infinite scroll can provide a more user-friendly experience.","ref":"load_more_and_infinite_scroll.html#final-thoughts"},{"type":"extras","title":"Resources - Load More and Infinite Scroll","doc":"- [Infinite Scrolling & Role=Feed Accessibility Issues](https://www.deque.com/blog/infinite-scrolling-rolefeed-accessibility-issues/)\n- [Infinite Scrolling: When to Use It, When to Avoid It](https://www.nngroup.com/articles/infinite-scrolling-tips/)\n- [Humane by Design Principles: Finite](https://humanebydesign.com/principles/finite)\n- [Infinite Scroll & Accessibility! Is It Any Good?](https://www.digitala11y.com/infinite-scroll-accessibility-is-it-any-good/)\n- [OK ARIA! Role=feed is here & it’s not ready for prime time](https://www.digitala11y.com/ok-aria-rolefeed-is-here-its-not-ready-for-prime-time/)\n- [So You Think You’ve Built a Good Infinite Scroll](https://adrianroselli.com/2014/05/so-you-think-you-built-good-infinite.html)","ref":"load_more_and_infinite_scroll.html#resources"},{"type":"extras","title":"Page Size Control","doc":"# Page Size Control\n\nTo allow users to change the page size, you can create a component similar to\nthis:\n\n```elixir\nattr :current_size, :integer, required: true\n\ndef page_size_links(assigns) do\n  ~H\"\"\"\n   \n     \n      <.link phx-click=\"set-page-size\" phx-value-size={page_size}>\n        {page_size}\n      </.link>\n     \n   \n  \"\"\"\nend\n```\n\nRender it with:\n\n```heex\n<.page_size_links current_size={@meta.page_size} />\n```\n\nAnd handle the event:\n\n```elixir\n  def handle_event(\"set-page-size\", %{\"size\" => page_size}, socket) do\n    flop = %{socket.assigns.meta.flop | page_size: page_size, limit: nil}\n    path = Flop.Phoenix.build_path(~p\"/pets\", flop)\n\n    {:noreply, push_patch(socket, to: path)}\n  end\n```","ref":"page_size_control.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}